C
C      2. SUBROUTINES FOR CALCULATION OF ELECTRON DENSITY PROFILE
C
      REAL FUNCTION XXE(H)
C     CALCULATION OF ELECTRON DENSITY PROFILE AT ALTITUDE FROM
C     AH TO 1000 KM
      REAL NMF2
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK3/H1
      COMMON /BLKS/Y1,Y11,H2,Y2,Y21,KEY,X0
      IF(H.LT.HMF2) GOTO 100
      XXE=XE1(H)
      RETURN
  100 IF(H.LT.HMF1) GOTO 200
      XXE=XE2(H)
      RETURN
  200 IF(H.LT.H2) GOTO 300
      XXE=XE3(H)
      RETURN
  300 IF (H.LE.H1) GO TO 400
      XXE=XE4(H)
      RETURN
  400 XXE=XE5(H)
      RETURN
      END
C
      FUNCTION XE1(H)
C     TOPSIDE IONOSPHERE ELECTRON DENSITY PROFILE
C     (HMF2....1000 KM)
      REAL NMF2,DH(12),DN(12),DNDH(12)
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK/DH,DN,DNDH,KT
      X=H-HMF2
      XNE=0.
      DO 1 I=2,KT
      V=(X-DH(I))*0.1
      IF (I.LE.3) GO TO 2
      XNE=XNE+(DNDH(I)-DNDH(I-1))*10.*ALOG(1.+EXP(V))
      GO TO 1
    2 VS=(1.+EXP(V))/(1.+EXP((DH(1)-DH(I))*0.1))
      XNE=XNE+(DNDH(I)-DNDH(I-1))*10.*ALOG(VS)
    1 CONTINUE
      XE1=NMF2*10.**XNE
      RETURN
      END
C
      REAL FUNCTION XE2(H)
C     SUB-PEAK ELECTRON DENSITY PROFILE IN THE F REGION
C     (HMF1....HMF2)
      REAL NMF2
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK2/B0,B1,C1
      X=(HMF2-H)/B0
      XE2=NMF2*EXP(-X**B1)/COSH(X)
      RETURN
      END
C
      REAL FUNCTION XE3(H)
C     F1 LAYER ELECTRON DENSITY PROFILE (HZ....HMF1)
      REAL NMF2
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK2/B0,B1,C1
      XE3=XE2(H)+NMF2*C1*SQRT(ABS(HMF1-H)/B0)
      RETURN
      END
C
      REAL FUNCTION XE4(H)
C     ELECTRON DENSITY PROFILE IN THE TRANSITION REGION
C     (H1....H2)
      COMMON /BLKS/Y1,Y11,H2,Y2,Y21,KEY,X0/BLOCK3/H1
      Z01=X0-H1
      Z02=X0-H2
      ZH1=H-H1
      ZH2=H-H2
      Z21=H2-H1
      IF (KEY.GT.0) GO TO 30
      DELT0=0.5*Z01*((Y2-Y1)/Z21-Y11)
      IF (H.GT.X0) GO TO 20
      Y=Y1+Y11*ZH1+DELT0*ZH1*ZH1/(Z01*Z01)
      XE4=EXP(Y)
      RETURN
   20 Y=Y2+Y21*ZH2+DELT0*ZH2*ZH2/(Z02*Z02)
      XE4=EXP(Y)
      RETURN
   30 ZZ2=(H1+H2)*0.5
      IF (H.LE.ZZ2) F=2.*ZH1*ZH1/(Z21*Z21)
      IF (H.GT.ZZ2) F=1.-2.*ZH2*ZH2/(Z21*Z21)
      Y=(1.-F)*(Y1+Y11*ZH1)+F*(Y2+Y21*ZH2)
      XE4=EXP(Y)
      RETURN
      END
C
      REAL FUNCTION XE5(X)
C     ELECTRON DENSITY PROFILE BELOW 200 KM
      REAL*8 SY(10),FK(10),FKW(10),A(13)
      COMMON /B5/FKW,NB/B6/HMLAT,H0,GLT,FG1,FG2/B7/A
     */BL10/HX1,HKX1,HX2,HKX2,HX3,HKX3,HKI1,HKI2
      DATA SY /24.06, 16.92, -197.27, 1776.79, -9248.86, 26505.61,
     1-43492.63, 40981.35, -20682.08, 4339.91/
      DATA FK /20.01, 42.97, -308.23, 1742.92, -6407.71, 14695.58,
     1-20918.81, 17973.78, -8537.24, 1721.32/
      HK=HKX1
      IF ((X.GT.HX1).AND.(X.LE.HX2)) HK=HKI1*(X-HX1)+HKX1
      IF (X.GT.HX2) HK=HKI2*(X-HX2)+HKX2
      IF (ABS(HMLAT).LE.FG1) GO TO 1
      SYN=REAX(10,X,H0,SY)
      SYN=EXP(SYN)
      FKN=REAX(10,X,H0,FK)
      FKN=EXP(FKN)
      FKNR=1.2*FKN
      IF (NB.LT.1) GO TO 2
      FKNR=REAX(10,X,H0,FKW)
      FKNR=EXP(FKNR)
    2 ENE=REAX(12,X,H0,A)
      ENE=EXP(ENE)*HK
      XE5=ENOYS(SYN,FKN,FKNR,ENE,X)
      GO TO 3
    1 ENE=REAX(12,X,H0,A)
      XE5=EXP(ENE)*HK
    3 RETURN
      END
C
C       3. SUBROUTINES FOR CALCULATION OF ELECTRON TEMPERATURE PROFILE
C
      SUBROUTINE TE(XMLAT,HLT,SR,SS,H,T1,T2)
      IF (H.LT.160.) GO TO 1
      CALL TEGT(H,T1)
      T2=-1.
      IF ((H.LE.200.).AND.(ABS(XMLAT).GE.65.)) CALL FITE(XMLAT,HLT,SR,
     *SS,H,X,T2)
      GO TO 2
    1 CALL FITE(XMLAT,HLT,SR,SS,H,T1,T2)
    2 RETURN
      END
C
      SUBROUTINE FITE(XMLAT,HOUR,SR,SS,H,TE1,TE2)
      REAL*8 T1(6),T2(6),TN1(6),TD1(6),TN2(6),TD2(6)
      COMMON /B4/T1,T2
      DATA TN1 /527.06,1551.08,-6190.70,13316.07,-11231.45,3204.29/
      DATA TD1 /701.24, 235.72, 972.71, -3636.46, 7374.63, -4054.11/
      DATA TN2 /1029.53,1799.37,-6077.71,12643.51,-7444.53,363.33/
      DATA TD2 /1058.8, 999.6, 323.68, -7684.13, 15831.40, -8409.56/
      IF (ABS(XMLAT).GT.54.) GO TO 1
      TE1=REAX(6,H,100.,T1)
      TE2=REAX(6,H,120.,T2)
      IF (H.LT.155.) GO TO 4
      TE1=(TE1+TE2)/2.
      TE2=-1.
      GO TO 4
    1 IF (ABS(XMLAT).GE.65.) GO TO 3
      TE1=REAX(6,H,100.,T1)
      TEN1=REAX(6,H,100.,TN1)
      TED1=REAX(6,H,100.,TD1)
      TE=HPOL(HOUR,TED1,TEN1,SR,SS)
      IF (H.LT.155.) GO TO 2
      TE2=REAX(6,H,120.,T2)
      TE1=(TE1+TE2)/2.
    2 TE1=TE1+(TE-TE1)*(ABS(XMLAT)-54.)/11.
      TE2=-1.
      GO TO 4
    3 TEN1=REAX(6,H,100.,TN1)
      TED1=REAX(6,H,100.,TD1)
      TE1=HPOL(HOUR,TED1,TEN1,SR,SS)
      TEN2=REAX(6,H,100.,TN2)
      TED2=REAX(6,H,100.,TD2)
      TE2=HPOL(HOUR,TED2,TEN2,SR,SS)
    4 RETURN
      END
C
      SUBROUTINE TEGT(H,TE)
      REAL M(10),D(9),HP(9),TEP(9)
      COMMON /GTE/M,D,HP,TEP/DM/NI
      TE=TEP(1)+M(2)*(H-HP(1))
      DO 1 I=2,NI
      V=(H-HP(I))/D(I)
      IF(V.GT.174.) GO TO 2
      TE=TE+(M(I+1)-M(I))*D(I)*ALOG((1+EXP(V))/
     /   (1+EXP((HP(1)-HP(I))/D(I))))
      GO TO 3
    2 TE=TE+(M(I+1)-M(I))*(H-HP(I))
    3 CONTINUE
    1 CONTINUE
      RETURN
      END
C
      SUBROUTINE TEDAT(TEINP)
C     TE COEFFICIENTS FOR TIME LT (I): 0,2,...22h;
C     LATI  (K) : -60,-40,-20,-10, 0, 10, 20, 40, 60;
C     LONGI (L) : 30, 150, 270; SEASONS (J) : 1,2,3.
      DIMENSION TEINP (12,9,3,3)
      REAL TEW1(12,9),TEW2(12,9),TEW3(12,9),TEW4(12,9),TEW5(12,9),
     &TEW6(12,9),TEW7(12,9),TEW8(12,9),TEW9(12,9)
      DATA TEW1/3140.,2960.,3460.,4340.,4570.,4820.,4840.,5040.,5040.,
     &4640.,4370.,3670.,3140.,2960.,3460.,4340.,4570.,4820.,4840.,5040.,
     &5040.,4640.,4370.,3670.,2660.,2470.,2890.,4250.,4720.,4880.,4900.,
     &5120.,5220.,4680.,4140.,3230.,2300.,2160.,2330.,4250.,4730.,4720.,
     &4740.,4720.,5080.,4690.,3730.,2790.,1940.,1850.,1770.,4240.,4730.,
     &4550.,4570.,4310.,4930.,4690.,3310.,2340.,1950.,1910.,1770.,4180.,
     &4810.,4580.,4450.,4380.,4860.,4680.,3400.,2380.,1950.,1960.,1760.,
     &4110.,4880.,4610.,4320.,4450.,4790.,4670.,3490.,2410.,3280.,3350.,
     &2680.,3660.,5260.,4980.,4820.,4560.,4880.,5330.,4550.,3790.,3960.,
     &3830.,3550.,3850.,4510.,4530.,4340.,3980.,4220.,4790.,4690.,4330./
      DATA TEW2/3980.,3990.,4070.,4120.,4120.,4120.,4130.,4170.,4220.,
     &4230.,4170.,4050.,3570.,3460.,3940.,4390.,4410.,4620.,4640.,4780.,
     &4800.,4600.,4480.,4000.,2240.,2080.,2360.,4150.,4840.,4820.,4850.,
     &5000.,5310.,4730.,3840.,2800.,2090.,1970.,2050.,4230.,4770.,4680.,
     &4680.,4620.,5070.,4690.,3560.,2560.,1940.,1860.,1730.,4300.,4690.,
     &4530.,4500.,4240.,4830.,4650.,3270.,2320.,1950.,1910.,1750.,4210.,
     &4780.,4570.,4410.,4350.,4810.,4660.,3380.,2370.,1950.,1960.,1760.,
     &4110.,4870.,4610.,4320.,4450.,4790.,4670.,3480.,2410.,2900.,3000.,
     &2420.,3640.,5280.,4940.,4750.,4640.,4950.,5260.,4330.,3440.,4060.,
     &3950.,3540.,3860.,4660.,4660.,4470.,4060.,4320.,4950.,4810.,4450./
      DATA TEW3/2820.,2620.,3090.,4290.,4680.,4900.,4920.,5160.,5190.,
     &4660.,4250.,3400.,2110.,1970.,2140.,4140.,4840.,4730.,4770.,4790.,
     &5240.,4740.,3650.,2620.,1940.,1850.,1740.,4270.,4710.,4530.,4540.,
     &4250.,4880.,4680.,3270.,2320.,1930.,1880.,1740.,4240.,4750.,4550.,
     &4420.,4320.,4810.,4640.,3340.,2340.,1910.,1900.,1730.,4210.,4790.,
     &4570.,4290.,4380.,4740.,4600.,3400.,2350.,2260.,2310.,1980.,3920.,
     &5050.,4740.,4500.,4540.,4870.,4900.,3790.,2760.,2600.,2720.,2220.,
     &3620.,5300.,4900.,4700.,4700.,5000.,5200.,4170.,3160.,4100.,3960.,
     &3530.,3860.,4710.,4700.,4470.,4080.,4360.,5010.,4850.,4490.,2910.,
     &2860.,2950.,3140.,3380.,3480.,3480.,3430.,3460.,3470.,3330.,3100./
      DATA TEW4/3920.,3900.,3200.,3780.,5080.,4940.,4780.,4340.,4660.,
     &5300.,4870.,4370.,3920.,3900.,3200.,3780.,5080.,4940.,4780.,4340.,
     &4660.,5300.,4870.,4370.,3320.,3390.,2700.,3660.,5260.,4990.,4830.,
     &4560.,4880.,5340.,4570.,3830.,2630.,2670.,2230.,3910.,5050.,4800.,
     &4570.,4490.,4830.,4990.,4010.,3110.,1930.,1940.,1750.,4150.,4840.,
     &4600.,4310.,4420.,4770.,4640.,3450.,2390.,1940.,1900.,1770.,4190.,
     &4800.,4580.,4450.,4380.,4870.,4670.,3390.,2370.,1950.,1850.,1790.,
     &4230.,4750.,4560.,4580.,4340.,4960.,4700.,3330.,2350.,2630.,2440.,
     &2850.,4240.,4730.,4870.,4900.,5110.,5230.,4680.,4120.,3200.,3720.,
     &3680.,4100.,4380.,4330.,4490.,4500.,4600.,4660.,4560.,4470.,4090./
      DATA TEW5/2870.,2820.,2910.,3100.,3300.,3400.,3420.,3390.,3400.,
     &3380.,3240.,3030.,4120.,4020.,3530.,3870.,4740.,4720.,4540.,4100.,
     &4370.,5040.,4870.,4510.,2600.,2720.,2220.,3620.,5300.,4900.,4700.,
     &4700.,5000.,5200.,4170.,3160.,2260.,2310.,1970.,3940.,5030.,4730.,
     &4510.,4520.,4870.,4900.,3770.,2750.,1920.,1890.,1720.,4250.,4750.,
     &4560.,4320.,4340.,4730.,4590.,3360.,2340.,1940.,1870.,1760.,4240.,
     &4750.,4560.,4450.,4340.,4850.,4650.,3350.,2350.,1950.,1850.,1790.,
     &4230.,4750.,4560.,4580.,4340.,4960.,4700.,3330.,2350.,2410.,2240.,
     &2580.,4190.,4790.,4840.,4870.,5050.,5270.,4710.,3960.,2980.,3620.,
     &3540.,4160.,4380.,4380.,4580.,4590.,4720.,4740.,4580.,4480.,4040./
      DATA TEW6/3600.,3650.,2890.,3680.,5240.,5020.,4880.,4500.,4830.,
     &5390.,4730.,4090.,2320.,2480.,2070.,3720.,5220.,4820.,4580.,4670.,
     &4960.,5050.,3940.,2860.,1910.,1900.,1730.,4210.,4790.,4570.,4290.,
     &4380.,4740.,4600.,3400.,2350.,1930.,1880.,1740.,4240.,4750.,4570.,
     &4420.,4320.,4810.,4640.,3340.,2340.,1940.,1850.,1740.,4270.,4710.,
     &4530.,4540.,4250.,4880.,4680.,3270.,2320.,2090.,1970.,2050.,4210.,
     &4780.,4680.,4700.,4630.,5100.,4710.,3560.,2560.,2240.,2080.,2360.,
     &4150.,4840.,4820.,4850.,5000.,5310.,4730.,3840.,2800.,3590.,3490.,
     &3960.,4390.,4400.,4610.,4620.,4750.,4780.,4590.,4480.,4020.,3970.,
     &3980.,4080.,4140.,4130.,4130.,4140.,4180.,4240.,4260.,4190.,4060./
      DATA TEW7/3910.,3080.,3440.,4720.,4760.,4940.,4550.,4550.,5200.,
     &4980.,4320.,4380.,3910.,3080.,3440.,4720.,4760.,4940.,4550.,4550.,
     &5200.,4980.,4320.,4380.,3440.,2550.,2760.,4630.,4850.,5030.,4730.,
     &4680.,5280.,5060.,4050.,3850.,2720.,2210.,2220.,4520.,4790.,4780.,
     &4630.,4500.,5020.,4860.,3640.,3110.,2000.,1860.,1670.,4400.,4720.,
     &4530.,4530.,4310.,4750.,4650.,3230.,2360.,2015.,1860.,1680.,4400.,
     &4720.,4540.,4540.,4330.,4750.,4650.,3220.,2370.,2030.,1860.,1680.,
     &4400.,4710.,4540.,4550.,4340.,4750.,4640.,3220.,2380.,3400.,2530.,
     &2730.,4620.,4850.,5030.,4740.,4680.,5270.,5050.,4030.,3840.,3980.,
     &3600.,3960.,4320.,4290.,4470.,4280.,4270.,4550.,4280.,4030.,4250./
      DATA TEW8/3510.,3560.,3670.,3800.,3940.,4070.,4130.,4060.,3890.,
     &3690.,3550.,3500.,4080.,3500.,3900.,4540.,4480.,4660.,4350.,4370.,
     &4840.,4580.,4240.,4460.,2830.,2090.,2110.,4380.,4780.,4940.,4830.,
     &4700.,5090.,4870.,3600.,3130.,2390.,1990.,1900.,4450.,4770.,4720.,
     &4650.,4470.,4920.,4780.,3430.,2730.,1940.,1890.,1680.,4520.,4750.,
     &4490.,4460.,4240.,4750.,4680.,3250.,2330.,1990.,1880.,1680.,4460.,
     &4730.,4520.,4500.,4290.,4750.,4660.,3240.,2360.,2030.,1860.,1680.,
     &4400.,4710.,4540.,4550.,4340.,4750.,4640.,3220.,2380.,3080.,2280.,
     &2380.,4480.,4810.,4980.,4790.,4690.,5170.,4950.,3790.,3430.,4040.,
     &3540.,3920.,4460.,4420.,4590.,4320.,4330.,4740.,4480.,4170.,4380./
      DATA TEW9/3670.,2730.,3020.,4730.,4880.,5070.,4690.,4670.,5350.,
     &5130.,4230.,4140.,2530.,1960.,1900.,4320.,4720.,4800.,4770.,4660.,
     &4940.,4750.,3450.,2870.,1950.,1870.,1670.,4470.,4730.,4500.,4490.,
     &4270.,4740.,4660.,3230.,2330.,1950.,1870.,1670.,4470.,4730.,4500.,
     &4490.,4270.,4740.,4660.,3230.,2330.,1950.,1870.,1670.,4470.,4730.,
     &4500.,4490.,4270.,4740.,4660.,3230.,2330.,2390.,1980.,1890.,4430.,
     &4760.,4720.,4660.,4490.,4920.,4770.,3420.,2730.,2830.,2090.,2110.,
     &4380.,4780.,4940.,4830.,4700.,5090.,4870.,3600.,3130.,4040.,3510.,
     &3910.,4500.,4460.,4630.,4340.,4350.,4800.,4540.,4220.,4430.,3520.,
     &3560.,3680.,3820.,3950.,4100.,4160.,4090.,3920.,3690.,3540.,3520./
      J=0
12    J=J+1
      L=0
11    L=L+1
      K=0
20    K=K+1
      DO 10 I=1,12
C      IK=I+(K-1)*12
      IF (J-2) 30,40,50
 30   IF (L-2) 31,32,33
 31   TEINP(I,K,L,J)=TEW1(I,K)
      GO TO 10
 32   TEINP(I,K,L,J)=TEW2(I,K)
      GO TO 10
 33   TEINP(I,K,L,J)=TEW3(I,K)
      GO TO 10
 40   IF (L-2) 41,42,43
 41   TEINP(I,K,L,J)=TEW4(I,K)
      GO TO 10
 42   TEINP(I,K,L,J)=TEW5(I,K)
      GO TO 10
 43   TEINP(I,K,L,J)=TEW6(I,K)
      GO TO 10
 50   IF (L-2) 51,52,53
 51   TEINP(I,K,L,J)=TEW7(I,K)
      GO TO 10
 52   TEINP(I,K,L,J)=TEW8(I,K)
      GO TO 10
 53   TEINP(I,K,L,J)=TEW9(I,K)
 10   CONTINUE
      IF (K.LT.9) GO TO 20
      IF (L.LT.3) GO TO 11
      IF (J.LT.3) GO TO 12
      RETURN
      END
C
C      4. SUBROUTINES FOR CALCULATION OF ION TEMPERATURE PROFILE
C
      FUNCTION TUNCAL(COV,XLATI,SD,SLSTA)
C     CALCULATION OF EXOSPHERIC TEMPERATURE WITH JACCIA-77 MODEL FOR GIVEN
C     INDEX F10.7 (COV), LATITUDE (XLATI), SUN DECLINATION
C     (SD), SOLAR ZENITH ANGLE FOR GIVEN HOUR (SLSTA)
      UMR=0.01745329
      T12=5.48*COV**0.8+101.8*COV**0.4
      DN=COS(XLATI*XLATI/90.0*UMR)
      DN=2.0+DN*DN
      HBET=(SLSTA-60.0)*UMR
      FH=ABS(COS(0.5*HBET))**DN+0.08*COS(3.0*HBET-1.309)
      TUNCAL=1.0+0.15*SD*SIN(XLATI*UMR)/23.44
      TUNCAL=T12*(TUNCAL+0.24*COS(XLATI*UMR)*(FH-0.5))
      RETURN
      END
C
      REAL FUNCTION TN(H,ATN,CTN,TNX,DSL,DGT)
C     CALCULATION OF NEUTRAL GAS TEMPERATURE WITH MODEL JACCIA-77
      DIMENSION CTN(2),ATN(2)
      IF(H.GT.90.0) GO TO 10
      TN=188.0
      GO TO 30
   10 ZX=H-125.0
      ZZ=(H-90.0)*(H-90.0)
      ZX2=ZX*ZX
      IF (H.GE.125.0) GO TO 20
      X=CTN(1)*ZX
      TN=TNX+ATN(1)*ATAN(X*(1.0+1.7*ZX2/ZZ))
      GO TO 30
   20 X=CTN(2)*ZX
      TN=TNX+ATN(2)*ATAN(X*(1.0+5.5E-05*ZX2))
   30 DSLT=0.0
      IF (H.GT.200.0) GO TO 40
      DH=H-102.5
      DSLT=-2.9*DSL*DH*EXP(-7.8E-05*ABS(DH)**2.7)
   40 X=0.006*(H-90.0)
      TN=TN+DGT*TANH(X)+DSLT
      RETURN
      END
C
      FUNCTION DTNDH(H,ATN,CTN,DSL,DGT)
C     CALCULATION OF DERIVATIVE FOR NEUTRAL GAS TEMPERATURE WITH HEIGHT
      DIMENSION CTN(2),ATN(2)
      IF(H.NE.90.0) GO TO 10
      DTNDH=0.0
      RETURN
   10 Z=(H-90.0)
      ZX=(H-125.0)
      ZZ=Z*Z
      ZX2=ZX*ZX
      IF (H.GE.125.0) GO TO 20
      Y=CTN(1)*ZX*(1.0+1.7*ZX2/ZZ)
      DTDH=ATN(1)/(1.0+Y*Y)*(Y/ZX+119.0*CTN(1)*ZX2/(ZZ*Z))
      GO TO 30
   20 Y=CTN(2)*ZX*(1.0+5.5E-05*ZX2)
      DTDH=ATN(2)/(1.0+Y*Y)*CTN(2)*(1.0+16.5E-05*ZX2)
   30 DSLDH=0.0
      IF (H.GT.200.0) GO TO 40
      DH=(H-102.5)
      Y=-7.8E-05*ABS(DH)**2.7
      DSLDH=-2.9*DSL*EXP(Y)*(1.0+2.7*Y*SIGN(1.0,DH))
   40 X=COSH(0.006*Z)
      DTNDH=DTDH+DSLDH+0.006*DGT/(X*X)
      RETURN
      END
C
      REAL FUNCTION TI(H)
C     CALCULATION OF ION TEMPERATURE PROFILE AT HEIGHTS FROM HS TO 1000 KM
      REAL MM,G(2)
      COMMON/BLOCK8/HS,TNHS,XSM(2),MM(3)
      G(1)=20.0
      G(2)=50.0
      SUM=MM(1)*(H-HS)+TNHS
      DO 100 I=1,2
      A=H-XSM(I)
      IF((H-XSM(I)).LT.(100.0*G(I))) A=G(I)*ALOG(1.0+
     &EXP((H-XSM(I))/G(I)))
      B=HS-XSM(I)
      IF((HS-XSM(I)).LT.(100.0*G(I))) B=G(I)*ALOG(1.0+
     &EXP((HS-XSM(I))/G(I)))
  100 SUM=SUM+(MM(I+1)-MM(I))*(A-B)
      TI=SUM
      RETURN
      END
C
      REAL FUNCTION TEDER(H)
C     THIS SUBROUTINE JOINTLY WITH SUBROUTINE REGFA12 ALLOWS TO FIND OUT
C     THE HEIGHT ABOVE WHICH TI DIFFERS FROM TN
      REAL MM,ATN(2),CTN(2)
      COMMON/BLOCK5/TNX,ATN,CTN,DSL,DGT/BLOCK8/HS,TNHS,XSM(2),MM(3)
      TNH=TN(H,ATN,CTN,TNX,DSL,DGT)
      DTDX=DTNDH(H,ATN,CTN,DSL,DGT)
      TEDER=DTDX*(XSM(1)-H)+TNH
      RETURN
      END
C
C      5. SUBROUTINES FOR CALCULATION OF PROFILE OF ELECTRON
C               COLLISION FREQUENCY
C
      REAL FUNCTION HNU(H,DAYNS,HNE,TE,HN120,T120)
      IF(H.LE.110.)GO TO 1
      IF((H.GT.110.).AND.(H.LT.120.))GO TO 3
      HNU=HNU2(TE,HNE,H)
      GO TO 2
    1 HNU=HNU1(H,DAYNS)
      GO TO 2
    3 HNU=HNU1(110.0,DAYNS)**(12.-H/10.)*HNU2(T120,HN120,
     *120.0)**(H/10.-11.)
    2 RETURN
      END
C
      REAL FUNCTION HNU1(H,DAYNS)
      DIMENSION AF(5),P(2),APR(200)
      COMMON /BLKK1/APR,LP,DELFI
      Z=(H-50.)/60.
      T1=DAYNS-1.
      N=(LP-1)*25
      DO 15 K=1,2
      DO 16 I=1,5
      L=N+5*I
      S=APR(L)
      DO 17 J=1,4
      M=L-J
   17 S=S*Z+APR(M)
   16 AF(I)=S
      P(K)=EXP(AF(1))+EXP(AF(2))*COS(1.72142E-2*(T1-AF(4)))
     *+EXP(AF(3))*COS(3.44284E-2*(T1-AF(5)))
      IF(LP.EQ.8)GO TO 40
   15 N=LP*25
      HNU1=(P(1)+(P(2)-P(1))*DELFI)*1.95E6
      RETURN
 40   HNU1=P(1)*1.95E6
      RETURN
      END
C
      REAL FUNCTION HNU2(XTE,HNE,H)
      DIMENSION O2(75),O(75),HN2(75),Z(75),TN(75)
      COMMON /NET/HN2,O2,O,TN/BKK/NS,Z
      XTN=PAR(TN,Z,NS,H)
      TE=XTE
      IF (XTE.LT.XTN)TE=XTN
      XN2=PAR(HN2,Z,NS,H)
      XO=PAR(O,Z,NS,H)
      XO2=PAR(O2,Z,NS,H)
      STE=SQRT(TE)
      DN2=2.33E-17*XN2*(1.-1.21E-04*TE)*TE
      DO2=1.82E-16*XO2*(STE+3.6E-02*TE)
      DO=2.8E-16*XO*STE
      HNU2=DN2+DO2+DO+54.0*HNE*1.0E-06/(TE*STE)
      RETURN
      END
C
C      6. SUBROUTINES FOR CALCULATION OF ADDITIONAL PARAMETERS
C
      SUBROUTINE CORDM(FIA,XLM,XMLAT,XLONG,FIMAG,XLMAG)
C ---------
C     TRANSFORMATION OF GEOGRAPHIC TO GEOMAGNETIC COORDINATES
C     USING TABLES OF HAKURA
      DIMENSION RFI(2),RLM(2),FI(72)
      FII=ABS(FIA)
      IF (FII.LE.20.) GO TO 4
      ILM=IFIX(XLM/5.)
      IFI=IFIX((90.-FII)/2.)+1
      A=90.-2.*(IFI-1)
      B=5.*ILM
      IF (FIA.LT.0.) ILM=ILM+73
      IF (ILM.EQ.0) GO TO 2
      DO 1 I=1,ILM
    1 READ (12) FI
    2 CONTINUE
      DO 3 I=1,2
      READ (12) FI
      IF ((FI(IFI).EQ.0.).OR.(FI(IFI+1).EQ.0)) GO TO 4
      RFI(I)=FI(IFI)-0.5*(FI(IFI+1)-FI(IFI))*(FII-A)
      DLA=FI(IFI+37)-FI(IFI+36)
      IF (ABS(DLA).LE.180.) GO TO 8
      IF (DLA.GT.0.) XDLA=DLA-360.
      IF (DLA.LT.0.) XDLA=DLA+360.
      DLA=XDLA
    8 RLM(I)=FI(IFI+36)-0.5*DLA*(FII-A)+1.0E-06
      IF ((RLM(I).GE.360.).OR.(RLM(I).LT.0.)) RLM(I)=ABS(ABS(RLM(I))-
     *360.)
    3 CONTINUE
      FIMAG=RFI(1)+0.2*(RFI(2)-RFI(1))*(XLM-B)
      DLA=RLM(2)-RLM(1)
      IF (ABS(DLA).LE.180.) GO TO 9
      IF (DLA.GT.0.) XDLA=DLA-360.
      IF (DLA.LT.0.) XDLA=DLA+360.
      DLA=XDLA
    9 XLMAG=RLM(1)+0.2*DLA*(XLM-B)+1.0E-06
      IF (FIA.LT.0.) FIMAG=-1.0*FIMAG
      IF ((XLMAG.GE.360.).OR.(XLMAG.LT.0.)) XLMAG=ABS(ABS(XLMAG)-360.)
      GO TO 5
    4 FIMAG=XMLAT
      XLMAG=XLONG
    5 REWIND 12
      RETURN
      END
C
      SUBROUTINE GGM(ART,XLG,BG,XLM,BM)
C     CALCULATION OF GEOMAGNETIC COORDINATES (XLM,BM) FROM GEOGRAPHIC      
C     (XLG,BG) IF ART=0 AND VICE VERSA AT ART=1
      INTEGER ART
      ZPI=6.2831853
      FAKTOR=0.0174533
      CBG=11.4*FAKTOR
      CI=COS(CBG)
      SI=SIN(CBG)
      IF(ART.EQ.0) GOTO 10
      CBM=COS(BM*FAKTOR)
      SBM=SIN(BM*FAKTOR)
      CLM=COS(XLM*FAKTOR)
      SLM=SIN(XLM*FAKTOR)
      SBG=SBM*CI-CBM*CLM*SI
      BG=ASIN(SBG)
      CBG=COS(BG)
      SLG=(CBM*SLM)/CBG
      CLG=(SBM*SI+CBM*CLM*CI)/CBG
      IF (ABS(CLG).GT.1.0) CLG=1.0*CLG/ABS(CLG)
      XLG=ACOS(CLG)
      IF(SLG.LT.0.0) XLG=ZPI-ACOS(CLG)
      BG=BG/FAKTOR
      XLG=XLG/FAKTOR
      XLG=XLG-69.8
      IF(XLG.LT.0.0) XLG=XLG+360.0
      GOTO 20
10    YLG=XLG+69.8
      CBG=COS(BG*FAKTOR)
      SBG=SIN(BG*FAKTOR)
      CLG=COS(YLG*FAKTOR)
      SLG=SIN(YLG*FAKTOR)
      SBM=SBG*CI+CBG*CLG*SI
      BM=ASIN(SBM)
      CBM=COS(BM)
      SLM=(CBG*SLG)/CBM
      CLM=(-SBG*SI+CBG*CLG*CI)/CBM
      IF (ABS(CLM).GT.1.0) CLM=1.0*CLM/ABS(CLM)
      XLM=ACOS(CLM)
      IF(SLM.LT.0.0) XLM=ZPI-ACOS(CLM)
      BM=BM/FAKTOR
      XLM=XLM/FAKTOR
20    RETURN
      END
C
      SUBROUTINE C6KOEF(FIELD)
C     [SHEIKH,1977], COEFFICIENTS G(144) FOR EPOCH 1973 FOR CALCULATION
C     OF MAGNETIC FIELD PARAMETERS WITH MODEL POGO 68/10
      DIMENSION FIELD (144)
      REAL FEL1 (72),FEL2(72)
      DATA FEL1/0.0, 0.1506723,0.0101742, -0.0286519, 0.0092606,
     & -0.0130846, 0.0089594, -0.0136808,-0.0001508, -0.0093977,
     & 0.0130650, 0.0020520, -0.0121956, -0.0023451, -0.0208555,
     & 0.0068416,-0.0142659, -0.0093322, -0.0021364, -0.0078910,
     & 0.0045586,  0.0128904, -0.0002951, -0.0237245,0.0289493,
     & 0.0074605, -0.0105741, -0.0005116, -0.0105732, -0.0058542,
     &0.0033268, 0.0078164,0.0211234, 0.0099309, 0.0362792,
     &-0.0201070,-0.0046350,-0.0058722,0.0011147,-0.0013949,
     & -0.0108838,  0.0322263, -0.0147390,  0.0031247, 0.0111986,
     & -0.0109394,0.0058112,  0.2739046, -0.0155682, -0.0253272,
     &  0.0163782, 0.0205730,  0.0022081, 0.0112749,-0.0098427,
     & 0.0072705, 0.0195189, -0.0081132, -0.0071889, -0.0579970,
     & -0.0856642, 0.1884260,-0.7391512, 0.1210288, -0.0241888,
     & -0.0052464, -0.0096312, -0.0044834, 0.0201764,  0.0258343,
     &0.0083033,  0.0077187/
      DATA FEL2/0.0586055,0.0102236,-0.0396107,
     & -0.0167860, -0.2019911, -0.5810815,0.0379916,  3.7508268,
     & 1.8133030, -0.0564250, -0.0557352, 0.1335347, -0.0142641,
     & -0.1024618,0.0970994, -0.0751830,-0.1274948, 0.0402073,
     &  0.0386290, 0.1883088,  0.1838960, -0.7848989,0.7591817,
     & -0.9302389,-0.8560960, 0.6633250, -4.6363869, -13.2599277,
     & 0.1002136,  0.0855714,-0.0991981, -0.0765378,-0.0455264,
     &  0.1169326, -0.2604067, 0.1800076, -0.2223685, -0.6347679,
     &0.5334222, -0.3459502,-0.1573697,  0.8589464, 1.7815990,
     &-6.3347645, -3.1513653, -9.9927750,13.3327637, -35.4897308,
     &37.3466339, -0.5257398,  0.0571474, -0.5421217,  0.2404770,
     & -0.1747774,-0.3433644, 0.4829708,0.3935944, 0.4885033,
     &  0.8488121, -0.7640999, -1.8884945, 3.2930784,-7.3497229,
     & 0.1672821,-0.2306652, 10.5782146, 12.6031065, 8.6579742,
     & 215.5209961, -27.1419220,22.3405762,1108.6394043/
      K=0
      DO 10 I=1,72
      K=K+1
      FIELD(K)=FEL1(I)
10    FIELD(72+K)=FEL2(I)
      RETURN
      END
C
      SUBROUTINE FIELDG(DLAT,DLONG,ALT,X,Y,Z,F,DIP,SMODIP)
C     THIS IS SPECIAL VERSION OF MODEL OF MAGNETIC FIELD POGO68/107
C     (IN GAUSS). F-FULL FIELD, Z-VERTICAL COMPONENT (DIRECTED
C     DAWNWARD), X AND Y-COMPONENTS IN EQUATORIAL PLANE
C     (X=0 IN LONGITUDE DIRECTION), DIP-MAGNETIC INCLINATION (IN
C     DEGR.), MODIP-MODIFIED MAGNETIC DECLINATION DUE TO K. RAWER
C     [METEOROLOGICAL AND ASTRONOMICAL INFLUENCES ON RADIO
C     WAVE PROPAGATION. PERGAMON PRESS,1963,ALSO RADIO SCI. 10,669,75]
C     INPUT: DLAT, DLONG-GEOGRAPHIC COORDINATES (IN DEGR.),
C     ALT-HEIGHT (IN KM). COEFFICIENTS ARRAY IS IN SUBROUTINE
C     C6KOEF.
      DIMENSION H(144),XI(3)
      COMMON/BLOCK4/G(144)
      RLAT=DLAT*0.0174533
      CT=SIN(RLAT)
      ST=COS(RLAT)
      NMAX=11
      D=SQRT(40680925.0-272336.0*CT*CT)
      RLONG=DLONG*0.0174533
      CP=COS(RLONG)
      SP=SIN(RLONG)
      ZZZ=(ALT+40408589.0/D)*CT/6371.2
      RHO=(ALT+40680925.0/D)*ST/6371.2
      XXX=RHO*CP
      YYY=RHO*SP
      RQ=1.0/(XXX*XXX+YYY*YYY+ZZZ*ZZZ)
      XI(1)=XXX*RQ
      XI(2)=YYY*RQ
      XI(3)=ZZZ*RQ
      IHMAX=NMAX*NMAX+1
      LAST=IHMAX+NMAX+NMAX
      IMAX=NMAX+NMAX-1
      DO 100 I=IHMAX,LAST
100   H(I)=G(I)
      DO 200 K=1,3,2
      I=IMAX
      IH=IHMAX
300   IL=IH-I
      F1=2.0/FLOAT(I-K+2)
      X1=XI(1)*F1
      Y1=XI(2)*F1
      Z1=XI(3)*(F1+F1)
      I=I-2
      IF((I-1).LT.0) GOTO 400
      IF((I-1).EQ.0) GOTO 500
      DO 600 M=3,I,2
      H(IL+M+1)=G(IL+M+1)+Z1*H(IH+M+1)+X1*(H(IH+M+3)-H(IH+M-1))-
     &Y1*(H(IH+M+2)+H(IH+M-2))
      H(IL+M)=G(IL+M)+Z1*H(IH+M)+X1*(H(IH+M+2)-H(IH+M-2))+
     &Y1*(H(IH+M+3)+H(IH+M-1))
  600 CONTINUE
  500 H(IL+2)=G(IL+2)+Z1*H(IH+2)+X1*H(IH+4)-Y1*(H(IH+3)+H(IH))
      H(IL+1)=G(IL+1)+Z1*H(IH+1)+Y1*H(IH+4)+X1*(H(IH+3)-H(IH))
  400 H(IL)=G(IL)+Z1*H(IH)+2.0*(X1*H(IH+1)+Y1*H(IH+2))
  700 IH=IL
      IF(I.GE.K) GOTO 300
  200 CONTINUE
      S=0.5*H(1)+2.0*(H(2)*XI(3)+H(3)*XI(1)+H(4)*XI(2))
      XT=(RQ+RQ)*SQRT(RQ)
      X=XT*(H(3)-S*XXX)
      Y=XT*(H(4)-S*YYY)
      Z=XT*(H(2)-S*ZZZ)
      F=SQRT(X*X+Y*Y+Z*Z)
      Z=-Z*CT-(Y*SP+X*CP)*ST
      DIP=ASIN(Z/F)
      SMODIP=ASIN(DIP/SQRT(DIP*DIP+ST))
    9 DIP=DIP*57.2957795
      SMODIP=SMODIP*57.2957795
      RETURN
      END
C
      REAL FUNCTION GAMMA1(SMODIP,SLAT,SLONG,UT,XHOUR,IHARM,NQ,K1,M,
     &MM,M3,SFE)
C     [SHEIKH,4.3.77]. CALCULATION OF RATIO (FOF2/M3000) USING CCIR MAPS
C     FOR NUMERICAL COEFFICIENTS SFE(M,2*IHARM+1).
C     NQ(K1) - TERMS OF SERIES NQ1,NQ2,NQ3,...OF EXPANSION ON HIGH ORDER
C     LATITUDES FOR EACH LONGITUDINAL HARMONICS (IHARM),
C     M=1+NQ1+2(NQ2+1)+2(NQ3+1)+... .
      DIMENSION NQ(K1),XSINX(13),COEF(100),C(12),S(12),SFE(M3)
      LOGICAL UT
      RD=57.2957795
      IF(UT) GOTO 100
      TIME=(15.0*XHOUR-180.0-SLONG)/RD
      GOTO 150
  100 TIME=(15.0*XHOUR-180.0)/RD
  150 S(1)=SIN(TIME)
      C(1)=COS(TIME)
      DO 250 I=2,IHARM
      C(I)=C(1)*C(I-1)-S(1)*S(I-1)
      S(I)=C(1)*S(I-1)+S(1)*C(I-1)
250   CONTINUE
      DO 300 I=1,M
      COEF(I)=SFE((I-1)*MM+1)
      DO 300 J=1,IHARM
      COEF(I)=COEF(I)+SFE((I-1)*MM+2*J)*S(J)+SFE((I-1)*MM+2*J+1)*C(J)
300   CONTINUE
      SUM=COEF(1)
      SS=SIN(SMODIP/RD)
      S3=SS
      XSINX(1)=1.0
      INDEX=NQ(1)
      DO 350 J=1,INDEX
      SUM=SUM+COEF(1+J)*SS
      XSINX(J+1)=SS
      SS=SS*S3
350   CONTINUE
      XSINX(NQ(1)+2)=SS*S3
      NP=NQ(1)+1
      SS=COS(SLAT/RD)
      S3=SS
      DO 400 J=2,K1
      S0=SLONG*FLOAT(J-1)/RD
      S1=COS(S0)
      S2=SIN(S0)
      INDEX=NQ(J)+1
      DO 450 L=1,INDEX
      NP=NP+1
      SUM=SUM+COEF(NP)*XSINX(L)*SS*S1
      NP=NP+1
      SUM=SUM+COEF(NP)*XSINX(L)*SS*S2
450   CONTINUE
      SS=SS*S3
400   CONTINUE
      GAMMA1=SUM
      RETURN
      END
C
      FUNCTION HPOL(XHOUR,TW,XNW,SA,SU)
C     [D.BILITZA,1978]. SUBROUTINE FOR TIME INTERPOLATION USING
C     EPSTEIN FUNCTION WITH WIDTH OF TRANSITION REGION OF 1 HOUR
C     (DAY-NIGHT FUNCTION). TW,NW-DAYTIME AND NIGHTTIME VALUES OF
C     PARAMETER SA, SU - SUNRISE AND SUNSET TIME, HOURS
      IF(ABS(SU-SA).GE.24.0) GOTO 100
      HPOL=XNW+(TW-XNW)/(1.0+EXP(-(XHOUR-SA)/1.0))+(XNW-TW)
     &/(1.0+EXP(-(XHOUR-SU)/1.0))
      GOTO 200
100   HPOL=XNW
      IF(SA.LT.1.0) HPOL=TW
200   RETURN
      END
C
C      7. SUBROUTINES FOR CALCULATION OF E AND F1 REGION
C                PEAK PARAMETERS
C
      SUBROUTINE PMRE(ENM,HME,FOE,KF,FUN)
      DIMENSION H(68),ENE(68)
      REAL*8 A(13)
      COMMON /B7/A/B6/HMLAT,H0,GLT,FG1,FG2
      KN=33
      IF ((KF.GT.0).AND.(ABS(HMLAT).GT.FG1)) KN=68
      H(1)=95.
      H(2)=96
      IF (KF.GT.0) GO TO 1
      ENE(1)=REAX(12,95.,H0,A)
      ENE(2)=REAX(12,96.,H0,A)
      GO TO 2
    1 ENE(1)=FUN(95.)
      ENE(2)=FUN(96.)
    2 CONTINUE
      DO 5 I=3,KN
      H(I)=H(I-1)+1.
      K=I
      IF (KF.GT.0) GO TO 3
      ENE(I)=REAX(12,H(I),H0,A)
      GO TO 4
    3 ENE(I)=FUN(H(I))
    4 IF ((ENE(I-2).GE.ENE(I-1)).AND.(ENE(I-2).GT.ENE(I))) GO TO 6
    5 CONTINUE
    6 ENM=ENE(K-2)
      IF (KF.EQ.0) ENM=EXP(ENM)
      HME=H(K-2)
      FOE=8.98E-06*SQRT(ENM)
      RETURN
      END
C
      REAL FUNCTION DTAL(HME,NME)
C     CALCULATION OF FULL ELECTRON DENSITY PROFILE AT THE RANGE OF
C     HEIGHT FROM AH TO 1000 KM
      REAL NME
      DTAL=HME
      X=HME+1.
      KT=200-IFIX(HME)
      DO 1 I=1,KT
      X=X+1.
      XNE=XE5(X)
      IF (XNE-NME) 100,110,120
  100 IF ((XNE.GE.XE5(X-1)).AND.(XNE.GE.XE5(X+1))) GO TO 110
    1 CONTINUE
      RETURN
  110 DTAL=X
      GO TO 3
  120 DTAL=X
      IF (ABS(XNE-NME).GE.1.) DTAL=X-0.5
      IF (ABS(DTAL-HME).LT.3.) DTAL=HME
    3 RETURN
      END
C
C
      SUBROUTINE ROGUL(DAY,XHI,GRO)
C ---------------------------------------------------------------------
C   CALCULATES RATIO H0.5/HMF2 FOR HALF-DENSITY POINT (NE(H0.5)=0.5*NMF2)
C   T.L. GULYAEVA, ADVANCES IN SPACE RESEARCH 7, #6, 39-48, 1987.
C
C   INPUT: IDAY - DAY OF YEAR
C   XHI - SOLAR ZENITH ANGLE [DEGREE]
C
C   OUTPUT: GRO RATIO OF HALF DENSITY HEIGHT TO F PEAK HEIGHT
C   SX SMOOTHLY VARYING SEASON PARAMTER (SX=1 FOR
C   DAY=1; SX=3 FOR DAY=180; SX=2 FOR EQUINOX)
C -----------------------------------------------------------------------
C
      SX=2.-COS(DAY*0.017214206)
      XS=(XHI-20.*SX)/15.
      GRO=0.8-0.2/(1.+EXP(XS))
c same as gro=0.6+0.2/(1+exp(-xs))
      RETURN
      END
C
      REAL FUNCTION FOF1ED(YLATI,R,COSXHI)
C     [D. BILITZA,1977], CALCULATION OF FOF1 FOR GIVEN MAGNETIC LATITUDE
C     (YLATI), SUNSPOT NUMBER (R), AND COS OF SOLAR ZENITH ANGLE
C     (COSXHI) [SEE E.D.DUCHARME ET.AL.,RADIO SCI.,8,10,837-
C     839]. GEOMAGNETIC LATITUDE IS REPLACED BY MAGNETIC LATI [R.EYFRIG,1979]
      XLATI=ABS(YLATI)
      F0=4.35+XLATI*(0.0058-(1.2E-4)*XLATI)
      F100=5.348+XLATI*(0.011-(2.3E-4)*XLATI)
      FS=F0+(F100-F0)*R/100.0
      XMUE=0.093+XLATI*(0.0046-(5.4E-5)*XLATI)+(3.0E-4)*R
      FOF1ED=FS*COSXHI**XMUE
      RETURN
      END
C
C      8. SUBROUTINES FOR CALCULATION OF THE F2 LAYER PEAK 
C                         PARAMETERS
C
      REAL FUNCTION HF2OUT(XMODIP,XLATI,XLONGI,XMAGBR,XR,HOUR,FOE,FOF2,
     &DAY,XMLAT,XM3000)
      DIMENSION MQM(7),XMM0(441),FM3N(9,49,2),FM3S(9,49,2)
      COMMON /BLOKB/FM3N,FM3S
      DATA MQM/6,7,5,2,1,0,0/
      R=200.*XR/(XR**8+2.56E18)**0.125
      ABML=ABS(XMLAT)
      YLATI=ABS(XLATI)
      IF ((XLATI.GE.0.).OR.((XMLAT+11.4).GE.0.)) GO TO 15
      DO 10 I=1,49
      DO 10 J=1,9
      K=J+9*(I-1)
   10 XMM0(K)=(FM3S(J,I,1)*(100.0-R)+FM3S(J,I,2)*R)/100.0
      XM3000=GAMMA1(XMODIP,YLATI,XLONGI,.FALSE.,HOUR,4,MQM,7,49,9,441,
     *XMM0)
      GO TO 30
   15 CONTINUE
      DO 20 I=1,49
      DO 20 J=1,9
      K=J+9*(I-1)
   20 XMM0(K)=(FM3N(J,I,1)*(100.0-R)+FM3N(J,I,2)*R)/100.0
      XM3000=GAMMA1(XMODIP,XLATI,XLONGI,.FALSE.,HOUR,4,MQM,7,49,9,441,
     *XMM0)
   30 HF2OUT=HMF2ED(XMAGBR,R,FOF2/FOE,XM3000,HOUR,DAY,ABML)
      RETURN
      END
C
      SUBROUTINE CCIRCA(IMONTH)
C     CALCULATION OF COEFFICIENTS FOR SERIES OF EXPANSION OF
C     FOF2(76,13) AND SM3000(49,9) FOR GIVEN SUNSPOT NUMBER    
C     R AND MONTH, USING SPECIAL CCIR COEFFICIENTS
      DIMENSION F2(13,76,2),FM3N(9,49,2),FM3S(9,49,2)
      COMMON /BLOKA/F2/BLOKB/FM3N,FM3S
      MON=IMONTH+6
      IF (MON.GT.12) MON=MON-12
      READ (16,rec=mon) F2,FM3S
      REWIND 16
      READ (16,rec=imonth) F2,FM3N
      REWIND 16
      RETURN
      END
C
      REAL FUNCTION HMF2ED(XMAGBR,R,X,XM3,HOUR,DAY,ABML)
C     [D. BILITZA] CALCULATION OF F REGION PEAK HEIGHT (HMF2) FOR GIVEN 
C     MAGNETIC LATITUDE (XMAGBR) AND SUNSPOT NUMBER (R) FROM 
C     COEFFICIENT M3000(XM3) AND RATIO FOF2/FOE.
C     [SEE D. BILITZA ET. AL. TELECOMM.J.,46,549-553,1979]
      COMMON /BLO11/XHI/B6/HMLAT,H0NE,GLT,FG1,FG2
      F1=(2.32E-3)*R+0.222
      F2=1.2-(1.16E-2)*EXP((2.39E-2)*R)
      F3=0.096*(R-25.0)/150.0
      XMIN=1.7
      IF (ABS(HMLAT).LT.64.) GO TO 1
      XZ=(XHI-65.)/6.
      XZ1=XZ*XZ
      XZ1=XZ1*XZ1
      RBA=(XZ1+16.)**0.25
      RB=0.25*XZ/RBA+1.95
      XZ=(XHI-74.)/6.
      XZ1=XZ*XZ
      XZ1=XZ1*XZ1
      RHA=(XZ1+16.)**0.25
      RH=0.2*XZ/RHA+1.70
      XMIN=(RB-RH)*(R-10.)/140.+RH
    1 IF (X.LT.XMIN) X=XMIN
      DELM=F1*(1.0-R/150.0*EXP(-XMAGBR*XMAGBR/1600.0))/(X-F2)+F3
      HMF2ED=1490./(XM3+DELM)-176.
      IF (ABML.LT.30.) HMF2ED=HMF2ED*DELH(HOUR,DAY,ABML)
      RETURN
      END
C
      FUNCTION DELH(ALST,TD,AFI)
      TD1=TD-14.
      IF(TD1.LE.0.) TD1=TD1+365.
      YD=(TD1+29.5)/30.5
      Y=YD
      IF(Y.GT.6.5)Y=13-Y
      A1=0.02+ZFUN(1.,-1.,(Y-2.9)/1.1)/8.+0.18*EFUN(Y,2.,0.6,2)
      A2=ZFUN(0.07,0.18,(YD-1.)/1.2)-ZFUN(1.,1.,(YD-13.)/0.6)*0.18
      A2=A2-0.08*EFUN(YD,9.,4.-YD/6.,2)
      A3=ZFUN(0.09,0.28,YD-1)-0.28*ZFUN(1.,1.,YD-13)
      A3=A3-0.26*EFUN(YD,7.7,2.7,2)-0.05*EFUN(YD,3.,0.5,2)
      A4=ZFUN(.14,-.12,(YD-3.7)/0.6)+ZFUN(1.,1.,(YD-10.1)/.4)*0.12
      A4=A4-ZFUN(1.,-1.,(Y-1.3)*5)*0.11+EFUN(YD,6.8,1.,2)/10.
      DELH=0.98+A1*EFUN(ALST,0.,1.8,2)+A2*EFUN(ALST,5.2,1.4,2)
      DELH=DELH-(0.6*A3+0.4*A1-A4)*EFUN(ALST,21.,1.3,2)
      DELH=DELH+(5.*A3+(A1-A3)*(ALST-19.))*
     1     ZFUN(1.,1.,(ALST-17.7)/0.6)/10.
      DELH=DELH+(1-DELH)*AFI**2*(90.-2.*AFI)/27000.
      RETURN
      END
C
      REAL FUNCTION F2OUT(XMODIP,XLATI,XLONGI,R,HOUR)
C     [D.BILITZA,20.8.78]. CALCULATION OF XHMF2 AND NMF2 USING
C     CCIR MAPS FOR FOF2 AND M3000.
      DIMENSION FF0(988),MQF(9),F2(13,76,2)
      COMMON /BLOKA/F2
      DATA MQF/11,11,8,4,1,0,0,0,0/
      DO 20 I=1,76
      DO 20 J=1,13
      K=J+13*(I-1)
   20 FF0(K)=(F2(J,I,1)*(100.0-R)+F2(J,I,2)*R)/100.0
      F2OUT=GAMMA1(XMODIP,XLATI,XLONGI,.FALSE.,HOUR,6,MQF,9,76,13,988,
     &FF0)
      RETURN
      END
C
      REAL FUNCTION F2NW(DAY,XMLAT,HOUR,XLATI,XLONGI,XMODIP,R)
      DIMENSION FG(2),D(2)
      RNAS=RN(DAY,XMLAT)
      IF (R.LE.RNAS) GO TO 2
      CALL FCOR(DAY,HOUR,FG)
      IF ((XMLAT.LT.(FG(1)-10.)).OR.(XMLAT.GT.(FG(2)+10.))) GO TO 2
      IF ((XMLAT.GT.(FG(1)+10.)).AND.(XMLAT.LT.(FG(2)-10.))) GO TO 1
      FMAX=F2OUT(XMODIP,XLATI,XLONGI,R,HOUR)
      FNAS=F2OUT(XMODIP,XLATI,XLONGI,RNAS,HOUR)
      DO 3 I=1,2
    3 D(I)=1.+EXP((FG(I)-XMLAT)/2.)
      F2NW=FMAX-(FMAX-FNAS)/D(1)+(FMAX-FNAS)/D(2)
      RETURN
    1 F2NW=F2OUT(XMODIP,XLATI,XLONGI,RNAS,HOUR)
      RETURN
    2 F2NW=F2OUT(XMODIP,XLATI,XLONGI,R,HOUR)
      RETURN
      END
C
      SUBROUTINE FCOR(DAY,HOUR,FGR)
      DIMENSION FWIN(25),FEQ(25),FSUM(25),FGR(2)
      DATA FWIN /7.,2.,4*0.,2.,7.,10.,15.,18.,20.,24.,25.,24.,22.,20.,
     115.,12.,11.,10.,9.,8.,2*7./, FEQ /60.,62.,65.,67.,69.,70.,73.,
     274.,4*75.,74.,72.,71.,70.,69.,67.,66.,65.,64.,62.,3*60./, FSUM
     3/-20.,-20.,-20.,-20.,-20.,-22.,-24.,-26.,-30.,-35.,-40.,-45.,-49.
     4,-50.,-50.,-46.,-40.,-35.,-30.,-24.,-20.,-15.,-15.,-17.,-20./
      IF ((DAY.LE.31.).OR.(DAY.GE.335.)) CALL SEAS(FWIN,HOUR,-68.,
     *130.,FGR)
      IF ((DAY.GT.31.).AND.(DAY.LT.90.)) CALL SEASM (FWIN,FEQ,HOUR,
     *-68.,130.,-49.,130.,DAY,FGR)
      IF ((DAY.EQ.90.).OR.(DAY.EQ.273.)) CALL SEAS (FEQ,HOUR,49.,130.,
     *FGR)
      IF ((DAY.GT.90.).AND.(DAY.LT.152.)) CALL SEASM (FEQ,FSUM,HOUR,
     *-49.,130.,130.,90.,DAY,FGR)
      IF ((DAY.GE.152.).AND.(DAY.LE.212.)) CALL SEAS(FSUM,HOUR,130.,
     *110.,FGR)
      IF ((DAY.GT.212.).AND.(DAY.LT.273.)) CALL SEASM (FSUM,FEQ,HOUR,
     *130.,90.,-49.,130.,DAY,FGR)
      IF ((DAY.GT.273.).AND.(DAY.LT.335.)) CALL SEASM (FEQ,FWIN,HOUR,
     *-49.,130.,-68.,130.,DAY,FGR)
      RETURN
      END
C
      SUBROUTINE SEAS (FMAS,HOUR,FGR1,FGR2,FGR)
      DIMENSION FMAS(25),FGR(2)
      IHOUR=IFIX(HOUR)
      FGR(1)=FGR1
      FGR(2)=FGR2
      I=2
      IF (FGR(2).LT.130.) I=1
      FGR(I)=FMAS(IHOUR+1)+(FMAS(IHOUR+2)-FMAS(IHOUR+1))*(HOUR-IHOUR)
      RETURN
      END
C
      SUBROUTINE SEASM(FMAS1,FMAS2,H,FGR1,FGR2,GR1,GR2,DAY,FGR)
      DIMENSION FMAS1(25),FMAS2(25),FM1(2),FM2(2),FGR(2)
      IH=IFIX(H)
      FM1(1)=FGR1
      FM1(2)=FGR2
      FM2(1)=GR1
      FM2(2)=GR2
      I=2
      IF (FM1(2).LT.130.) I=1
      FM1(I)=FMAS1(IH+1)+(FMAS1(IH+2)-FMAS1(IH+1))*(H-IH)
      K=2
      IF (FM2(2).LT.130.) K=1
      FM2(K)=FMAS2(IH+1)+(FMAS2(IH+2)-FMAS2(IH+1))*(H-IH)
      FK=SPRN(DAY)
      DO 1 I=1,2
    1 FGR(I)=FM1(I)+(FM2(I)-FM1(I))*FK
      RETURN
      END
C
      REAL FUNCTION RN(TD,FI)
      REAL A(3,4),A0(4)
      DATA A/124.,137.5,160.,22.,7.5,10.,-20.,-5.,-15.,14.,10.,6./
      DO 1 I=1,4
      SLG=(A(2,I)-A(1,I))*ZFUN(1.,1.,(TD-53.)/10.)
      SLG=SLG+(A(3,I)-A(2,I))*ZFUN(1.,1.,(TD-235.)/10.)
      A0(I)=A(1,I)+(SLG+(A(1,I)-A(3,I))*ZFUN(1.,1.,(TD-312.)/10.))/2.
    1 CONTINUE
      RN=ZFUN(A0(1),A0(2),(FI-A0(3))/A0(4))
      RETURN
      END
C
      REAL FUNCTION SPRN(DNR)
      SPRN=0
      IF ((DNR.GT.31.).AND.(DNR.LT.90.)) SPRN=(DNR-31.)/59.
      IF ((DNR.GT.90.).AND.(DNR.LT.152.)) SPRN=(DNR-90.)/62.
      IF ((DNR.GT.212.).AND.(DNR.LT.273.)) SPRN=(DNR-212.)/61.
      IF ((DNR.GT.273.).AND.(DNR.LT.335.)) SPRN=(DNR-273.)/62.
      RETURN
      END
C
      SUBROUTINE F0F2AN(FM0,LAT0,LONG0,TD0,UT0,RC,F107C,KP,KP1,F0F2)
C
C______ CALCULATION OF F2 LAYER CRITICAL FREQUENCY__________________________
C         AT GEOMAGNETIC LATITUDES
C                40-80 DEGR.
      INTEGER TD,TD1
      REAL LATI,LONGI,KP,M,K,KP1,KPP,LAT0,LONG0
      FM=ABS(FM0)
      LATI=ABS(LAT0)
      LONGI=LONG0
      TD2=TD0
      UT=UT0
         IF(FM0.GE.0.) GO TO 583
      LONGI=LONGI+180.
      IF(LONGI.GE.360.) LONGI=LONGI-360.
      TD2=TD2+183.
      IF(TD2.GE.366.) TD2=TD2-365.
      UT=UT+12.
      IF(UT.GE.24.) UT=UT-24.
  583 CONTINUE
      KPP=KP
      IF(KP1.GT.KP) KPP=KP1
       TD=INT(TD2)
      XHI0=LATI-ATAN(0.43*SIN(2*3.1416*(TD-81)/365.))*180./3.1416
      TD1=TD-15
        IF(TD1.LE.0) TD1=TD1+365
      CALL FACTOM(TD1,LATI,LONGI,M,N1,N2)
      CALL F0F20(F107C,RC,XHI0,M,TD1,N1,N2,F0F2,FM0)
      CALL COEFK(F107C,TD,KPP,FM,LATI,LONGI,UT,K)
      F0F2=F0F2*K
      RETURN
      END
C
      SUBROUTINE F0F20(F107C,RC,XZU,M,TD1,N1,N2,F0F2,FM0)
      REAL N,M
      REAL AK(12),BK(12),CK(12),DK(12)
      INTEGER TD1
      DATA AK/5.9,6.1,7.3,6.7,6.6,6.1,6.,6.,6.3,7.8,6.9,6.7/,
     *BK/2.78,2.88,2.17,2.61,2.07,1.96,1.94,2.07,2.38,1.87,2.43,2.31/,
     *CK/0.42,0.46,0.66,0.6,0.53,0.47,0.37,0.45,0.59,0.6,0.5,0.48/,
     *DK/-0.46E-3,-0.35E-3,-0.81E-3,1.43E-3,1.91E-3,3.4E-3,3.5E-3,
     *2.2E-3,0.8E-3,-1.2E-3,-1.1E-3,-0.98E-3/
      ZU=XZU
      IF (ZU.GT.99.0) ZU=99.0
      ST=(TD1-N1)*1./(N2-N1)
      L1=INT(N1/30.)+1
      L2=INT(N2/30.)+1
      IF(L2.GT.12) L2=1
         L1A=L1
         L2A=L2
         IF(FM0.GE.0.) GO TO 500
         L1A=L1+6
         L2A=L2+6
         IF(L1A.GT.12) L1A=L1A-12
         IF(L2A.GT.12) L2A=L2A-12
  500 AI=AK(L1A)+(AK(L2A)-AK(L1A))*ST
      BI=BK(L1A)+(BK(L2A)-BK(L1A))*ST
      CI=CK(L1)+(CK(L2)-CK(L1))*ST
      DI=DK(L1)+(DK(L2)-DK(L1))*ST
      A=AI+BI*AI*(ALOG10(F107C)-1.812)
      N=CI+DI*RC
      IF(ZU.LE.65.) FZU=1/COS(ZU*1.745E-2)
      IF(ZU.GT.65.) FZU=FCH(ZU)
      F0F2=A/FZU**N*M
      RETURN
      END
C
      SUBROUTINE COEFK(F107,TD1,KP,FM,FG,LM,UT,K)
      REAL KP,LM,K1,K2,K,LM1
      INTEGER N(8),TD1
      COMMON /BK/NBLOK/COR/FG1,FM1,LM1,UT1
      DATA N/32,60,91,135,213,244,274,319/
      FG1=FG
      FM1=FM
      LM1=LM
      UT1=UT
      NBLOK=0
      ALM=UT+LM1/15.
      IF (ALM.GE.24.) ALM=ALM-24.0
      ISE1=1
      IF ((TD1.LE.32).OR.(TD1.GE.319)) GO TO 6
      DO 1 I=2,8,2
      IF (TD1.GT.N(I)) GO TO 1
      ISE1=I/2
      IF (TD1.LE.N(I-1)) GO TO 6
      IF(I.GT.6) GO TO 2
      ISE2=I/2+1
      NUM1=N(I-1)
      NUM2=N(I)
      GO TO 4
    2 ISE1=1
      ISE2=4
      NUM1=N(I)
      NUM2=N(I-1)
    1 CONTINUE
    4 CALL SKSA(ISE1,F107,KP,K1)
      CALL SKSA(ISE2,F107,KP,K2)
      IF (ISE1.NE.1) GO TO 5
      CALL CORK(FM1,ALM,KP,K1)
    5 K=K1+(K2-K1)/(NUM2-NUM1)*(TD1-NUM1)
      GO TO 7
    6 CALL SKSA(ISE1,F107,KP,K)
      IF (ISE1.EQ.1) CALL CORK(FM1,ALM,KP,K)
    7 RETURN
      END
C
      SUBROUTINE SKSA(ISE,F107,KP,KSA)
      REAL KP,KSA,KSA1,KSA2
      IF (ISE.EQ.1.AND.KP.LE.4) GO TO 1
      CALL SKGA(ISE,2,KP,KSA)
      GO TO 2
    1 CALL SKGA(ISE,1,KP,KSA1)
      CALL SKGA(ISE,2,KP,KSA2)
      IF(F107.LE.90.) GO TO 3
      IF(F107.GE.200.) GO TO 4
      KSA=KSA1+(KSA2-KSA1)/110.*(F107-90.)
      GO TO 2
    3 KSA=KSA1
      GO TO 2
    4 KSA=KSA2
    2 RETURN
      END
C
      SUBROUTINE SKGA(ISE,ISA,KP,KGA)
      REAL KP,KGA
      IF(KP.GT.4.) GO TO 4
      IF(KP.GE.2.) GO TO 1
      IGA=1
      GO TO 6
    1 IGA=2
      GO TO 6
    4 IGA=3
    6 CALL SKUT(ISE,ISA,IGA,KGA)
    7 RETURN
      END
C
      SUBROUTINE SKUT (ISE,ISA,IGA,KU)
      INTEGER IS(14)
      REAL KU,KF(11,48),kfm(11,48,14)
      COMMON /BK/NBLOK/akfm/kfm
      DATA IS/111,112,121,122,123,221,222,223,321,322,323,421,422,423/
      ISV=ISE*100+ISA*10+IGA
    1 NBLOK=NBLOK+1
      IF(NBLOK.LE.14) GO TO 2
      WRITE (*,103) ISV
  103 FORMAT(10X,'TABLE OF COEFFICIENTS K IS MISSED'/10X,I4)
      KU=0.5
      return
    2 IF(ISV.NE.IS(NBLOK)) GO TO 1
      do 3 j=1,48
      do 3 i=1,11
    3 kf(i,j)=kfm(i,j,nblok)
      CALL SBLOK(KF,KU)
      RETURN
      END
C
      SUBROUTINE SBLOK(KF,KUT)
      REAL LMP,KIJ,KI1,K11,K1J
      REAL LM,LM1,LM2,LT,LMT(12),FTM(11),FTG(11),FT(11),KF(11,48)
      REAL KU(2),KUT
      COMMON /COR/FG,FM,LM,UT/BK/NBLOK
      DATA FTM/70.,68.,66.,64.,62.,60.,58.,56.,54.,52.,50./,
     *FTG/75.,70.,65.,60.,55.,50.,45.,40.,35.,30.,25./,
     *LMT/0.,30.,60.,90.,120.,150.,180.,210.,240.,270.,300.,330./
      DO 1 I=1,4
      IP=I-1
      UT2=6.0*I
      IF (UT.LE.UT2) GO TO 2
    1 CONTINUE
    2 UT1=UT2-6.0
      LT=UT+LM/15
      IF(LT.GE.24.) LT=LT-24.
      LM1=15*(LT-UT1)
      IF(LM1.LT.0.) LM1=360.+LM1
      LM2=15*(LT-UT2)
      IF (LM2.LT.0.) LM2=360.+LM2
      IF (NBLOK.GE.9.AND.NBLOK.LE.11) GO TO 4
      I1=1
      IF ((FM.GT.80.0).OR.(FM.LT.40.0)) GO TO 6
      DO 8 I=1,11
    8 FT(I)=FTM(I)
      FI=FM
      II=11
      GO TO 10
    4 I1=2
      IF (FG.GT.80.0) FG=80.0
      IF (FG.LT.25.0) FG=25.0
      DO 9 I=1,11
    9 FT(I)=FTG(I)
      FI=FG
      II=9
   10 IF ((FI.LE.FT(1)).AND.(FI.GE.FT(II))) GO TO 14
      I2=1
      I=1
      IF (FI.LT.FT(II)) I=II
      GO TO 150
   14 I2=2
      I=1
   16 I=I+1
      IF(FI.LE.FT(I)) GO TO 16
  150 L=1
      LMP=LM1
      UTP=UT1
   15 J=1
      IF(LMP.LE.330.) GO TO 17
      J2=12*(IP+L)+1
      J1=12*(IP+L-1)+1
      J=13
      GO TO 22
   17 J=J+1
      J1=12*(IP+L-1)+J
      IF(LMP.GT.LMT(J)) GO TO 17
      J2=J1
   22 IF(I2.EQ.2) GO TO 18
      KIJ=EXTRKF(KF,I,J1)
      KI1=EXTRKF(KF,I,J2-1)
      KU(L)=KI1+(KIJ-KI1)/30.*(LMP-LMT(J-1))
      GO TO 19
   18 AL=(FI-FT(I-1))/(FT(I)-FT(I-1))
      BETA=(LMP-LMT(J-1))/30.
      K11=EXTRKF(KF,I-1,J2-1)
      KI1=EXTRKF(KF,I,J2-1)
      K1J=EXTRKF(KF,I-1,J1)
      KIJ=EXTRKF(KF,I,J1)
      KU(L)=K11+AL*(KI1-K11)+BETA*(K1J-K11)+AL*BETA*(KIJ-KI1-K1J+K11)
   19 IF (ABS(UT-UTP).LE.2.0) GO TO 3
      IF (IP.EQ.3) IP=-1
      IF (L.EQ.2) GO TO 20
      L=2
      LMP=LM2
      UTP=UT2
      GO TO 15
   20 KUT=KU(1)+(KU(2)-KU(1))/2.0*(UT-UT1-2.0)
      GO TO 6
    3 KUT=KU(L)
    6 RETURN
      END
C
      FUNCTION EXTRKF(KF,I,J)
      REAL KF(11,48)
      IT=I
    3 IF(KF(IT,J).LE.0.015) GO TO 1
      EXTRKF=KF(IT,J)
      GO TO 2
    1 IT=IT+1
      GO TO 3
    2 RETURN
      END
C
      SUBROUTINE CORK(FT,LT,KP,K)
C-----CORRECTION OF COEFFICIENT  K--------------------------------------
C-----WINTER------------------------------------------------------------
C-----TROUGH------------------------------------------------------------
      REAL LT,KP,K
      IF((LT.GT.3.5).AND.(LT.LT.16.5)) RETURN
      IF (LT.GE.12.) GO TO 11
      T2=LT
      GO TO 12
  11  T2=LT-24.
  12  F0=65.2-2.1*KP-0.5*T2
      F=FT-F0
      IF((F.GE.9.).OR.(F.LE.-3.)) RETURN
      C18=(0.09+0.05*COS(0.314*(F-1.5)))*(1.+COS(1.26*(F-1.7)))+0.33
      C20=ZFUN(0.42,0.22,(F-4.)/1.1)+ZFUN(0.08,-0.08,F/0.9)
      C20=C20-0.17*EXP(-(F-6.7)**2)
      C22=ZFUN(0.31,0.11,F-2.5)+ZFUN(0.06,-0.06,(F+1.7)*2)
      C22=C22+0.22*EXP(-((F-7.5)/2.)**2)
      IF(KP.LE.2.) GO TO 1
      C22=C22+0.17*EXP(-((F-2.)/1.)**2)
   1  C00=0.24*SIN(0.365*(F-4.))+0.41
      IF(KP.LE.2.) GO TO 2
      C00=C00+ZFUN(0.13,0.13,(F-0.5)*4)*ZFUN(0.5,-0.5,(F-2.2)/0.6)
   2  C02=ZFUN(0.41,0.23,(F-5.0)/1.6)+ZFUN(0.04,-0.04,F+1.5)
      IF (KP.LE.2.) GO TO 3
      C02=C02+ZFUN(0.08,0.08,(F-3.)*2)*ZFUN(0.5,-0.5,(F-5.2)/0.4)
   3  IF(LT.LE.3.) GO TO 6
      IF(LT.GE.20.) GO TO 4
      C1=C18
      C2=C20
      T=18.
      GO TO 7
   4  IF(LT.GE.22.) GO TO 5
      C1=C20
      C2=C22
      T=20.
      GO TO 7
   5  C1=C22
      C2=C00
      T=22.
      GO TO 7
   6  C1=C00
      C2=C02
      T=0.0
   7  C=C1+(LT-T)*(C2-C1)/2.
      A=ZFUN(1.,-1.,(F-7.)*2.)*ZFUN(1.,1.,(F+2.)*2.)
      A=A*ZFUN(1.,-1.,(T2-2.75)*4.)*ZFUN(1.,1.,(T2+6.75)*4.)/16.
      K=C*A+K*(1-A)
      RETURN
      END
C
      FUNCTION FCH(ZU)
      VS=100.
      ZK=1.745E-2
      Y=SQRT(0.5*VS)*ABS(COS(ZU*ZK))
      IF(ZU-90.) 1,1,2
    1 FCH=SQRT(3.14159/2*VS)*EXP(Y**2)*ERFC(Y)
      GO TO 3
    2 A=VS*(1-SIN(ZU*ZK))
      A=SQRT(SIN(ZU*ZK))*EXP(A)
      A=A-0.5*EXP(Y**2)*ERFC(Y)
      FCH=A*SQRT(2*3.14159*VS)
    3 RETURN
      END
C
      REAL FUNCTION ERFC(Y)
C         SPECIFICATIONS FOR ARGUMENTS
      REAL               Y
C         SPECIFICATIONS FOR LOCAL VARIABLES
      INTEGER            ISW,I
      DIMENSION          P(3),Q(2),P1(5),Q1(4),P2(3),Q2(2)
      REAL               P,Q,P1,Q1,P2,Q2,XMIN,XLARGE,SQRPI,X,
     *                   RES,XSQ,XNUM,XDEN,XI,XBIG
C                                  COEFFICIENTS FOR 0.0 .LE. Y .LT.
C                                  .477
      DATA               P(1)/.3166529/,P(2)/1.722276/,
     *                   P(3)/21.38533/
      DATA               Q(1)/7.843746/,Q(2)/18.95226/
C                                  COEFFICIENTS FOR .477 .LE. Y
C                                  .LE. 4.0
      DATA               P1(1)/.5631696/,P1(2)/3.031799/,
     *                   P1(3)/6.865018/,P1(4)/7.373888/,
     *                   P1(5)/4.318779E-5/
      DATA               Q1(1)/5.354217/,Q1(2)/12.79553/,
     *                   Q1(3)/15.18491/,Q1(4)/7.373961/
C                                  COEFFICIENTS FOR 4.0 .LT. Y
      DATA               P2(1)/-5.168823E-2/,P2(2)/-.1960690/,
     *                   P2(3)/-4.257996E-2/
      DATA               Q2(1)/.9214524/,Q2(2)/.1509421/
C                                  CONSTANTS
      DATA               XMIN/1.0E-5/,XLARGE/4.1875E0/
C                                  ERFC(XBIG) .APPROX. SETAP
      DATA               XBIG/9.0/
      DATA               SQRPI/.5641896/
C                                  FIRST EXECUTABLE STATEMENT
      X = Y
      ISW = 1
      IF (X.GE.0.0E0) GO TO 5
      ISW = -1
      X = -X
    5 IF (X.LT..477E0) GO TO 10
      IF (X.LE.4.0E0) GO TO 25
      IF (ISW .GT. 0) GO TO 35
      IF (X.LT.XLARGE) GO TO 40
      RES = 2.0E0
      GO TO 55
C                                  ABS(Y) .LT. .477, EVALUATE
C                                  APPROXIMATION FOR ERFC
   10 IF (X.LT.XMIN) GO TO 15
      XSQ = X*X
      XNUM = (P(1)*XSQ+P(2))*XSQ+P(3)
      XDEN = (XSQ+Q(1))*XSQ+Q(2)
      RES = X*XNUM/XDEN
      GO TO 20
   15 RES = X*P(3)/Q(2)
   20 IF (ISW.EQ.-1) RES = -RES
      RES = 1.0E0-RES
      GO TO 55
C                                  .477 .LE. ABS(Y) .LE. 4.0
C                                  EVALUATE APPROXIMATION FOR ERFC
   25 XSQ = X*X
      XNUM = P1(5)*X+P1(1)
      XDEN = X+Q1(1)
      DO 30 I=2,4
         XNUM = XNUM*X+P1(I)
         XDEN = XDEN*X+Q1(I)
   30 CONTINUE
      RES = XNUM/XDEN
      GO TO 45
C                                  4.0 .LT. ABS(Y), EVALUATE
C                                  MINIMAX APPROXIMATION FOR ERFC
   35 IF (X.GT.XBIG) GO TO 50
   40 XSQ = X*X
      XI = 1.0E0/XSQ
      XNUM = (P2(1)*XI+P2(2))*XI+P2(3)
      XDEN = (XI+Q2(1))*XI+Q2(2)
      RES = (SQRPI+XI*XNUM/XDEN)/X
   45 RES = RES*EXP(-XSQ)
      IF (ISW.EQ.-1) RES = 2.0E0-RES
      GO TO 55
   50 RES = 0.0E0
   55 ERFC = RES
      RETURN
      END
C
      SUBROUTINE FACTOM(TD1,FI,LM,M,N1,N2)
      INTEGER ID(12),TD1
      REAL M,M1,M2,LM,LM1
      COMMON /A/FI1,LM1
      DATA ID/1,32,60,91,121,152,182,213,244,274,305,335/
      FI1=FI
      LM1=LM
      NT=0
    1 NT=NT+1
      IF(TD1.LT.ID(NT)) GO TO 2
      IF(TD1.LT.ID(12)) GO TO 1
      CALL INM(nt,M2)
      N1=ID(12)
      N2=ID(12)+31
      CALL INM(12,M1)
      GO TO 3
    2 CALL INM(nt-1,M1)
      N1=ID(NT-1)
      N2=ID(NT)
      CALL INM(nt,M2)
    3 M=M1+(M2-M1)/(N2-N1)*(TD1-N1)
      RETURN
      END
C
      SUBROUTINE INM(N,MP)
      REAL MF(17,36,12),MP,LM,LMV
      COMMON /A/FI,LM/AMF/MF
      IF(FI.LE.80.) GO TO 1
      FI=80.
    1 I=1
      J=1
      FIV=80.
      LMV=0.
    2 I=I+1
      FIV=FIV-5.
      IF(FI.GE.FIV) GO TO 3
      GO TO 2
    3 J=J+1
      LMV=LMV+10.
      IF(LM.GT.LMV) GO TO 3
      IF(LM.GT.350.) J1=1
      IF(LM.LE.350.) J1=J
      AL=(FI-(FIV+5.))/(-5.)
      BETA=(LM-(LMV-10.))/10.
      MP=MF(I-1,J-1,N)+AL*(MF(I,J-1,N)-MF(I-1,J-1,N))+
     +BETA*(MF(I-1,J1,N)-MF(I-1,J-1,N))+AL*BETA*(MF(I,J1,N)-
     -MF(I,J-1,N)-MF(I-1,J1,N)+MF(I-1,J-1,N))
    4 RETURN
      END
C
      SUBROUTINE SCAP(FM0,LT,UT0,KPP,KP1,F107,TD0,F0F2C)
C
C-----------CALCULATION OF F0F2 IN POLAR CAP ---------------------------
      REAL LT,LMLT,KP,KP1,KPP
      REAL NS0H,NS3H,NSKH,NSKF
      REAL NW0H,NW3H,NWKH,NWKF,NTKF
      FM=ABS(FM0)
      UT=UT0
      TD=TD0
        IF(FM0.GE.0.) GO TO 583
      UT=UT+12.
      IF(UT.GE.24.) UT=UT-24.
      TD=TD+183.
      IF(TD.GE.366.) TD=TD-365.
  583 CONTINUE
         KP=KPP
      IF(KP.LT.KP1) KP=KP1
           NS0H=0.
           NS3H=0.
           NW0H=0.
           NW3H=0.
      RAD=3.1416/180.
      TAT=90.-FM
      LMLT=15*LT
      HAD=3.1416/12.
C---------SUMMER--- B C A --- KP=0 ------------------------------------
      X=TAT*SIN(LMLT*RAD)
      Y=-TAT*COS(LMLT*RAD)
      SUT=SIN(UT*HAD)
      D=4.5-1.5*SUT
      C=27.5+2.5*SUT
      B=-5.-5.*SUT
      A=SQRT(X**2+(Y-B)**2)
      NS0H=7.6-2.8/(1+(A/C)**D)
C-------SUMMER--- B C A --- KP=3 --------------------------------------
      AL1=LMLT+22.5*(1.+SUT)
      AL2=LMLT+15.*(1.+SUT)
      X1=TAT*SIN(AL1*RAD)
      X2=TAT*SIN(AL2*RAD)
      Y1=-TAT*COS(AL1*RAD)
      Y2=-TAT*COS(AL2*RAD)
      X10=14.+SUT
      X20=-2.5*(5.+SUT)
      Y10=-1.5*(1.+SUT)
      Y20=-6.5-1.5*SUT
      GMA1=1.9+0.1*SUT
      GMA2=1.4+0.1*SUT
      A1=(X1-X10)**2+(Y1-Y10)**2/GMA1**2
      A2=(X2-X20)**2+(Y2-Y20)**2/GMA2**2
      NS3H=5.-(1.85+0.35*SUT)*EXP(-A1/144.)
      NS3H=NS3H-1.5*EXP(-A2/100.)
C-------WINTER--- B C A --- KP=0 --------------------------------------
      ALM=LMLT-25.
      X=TAT*SIN(ALM*RAD)
      Y=-TAT*COS(ALM*RAD)
      AN0A=0.167*Y+0.5*ABS(Y-2.)-
     -     0.3333*ABS(Y-5.)+2.67
      AN0B=0.361*Y+0.311*ABS(Y)+
     +     0.05*ABS(Y+10.)+1.5
      AN0=0.5*(AN0A+AN0B+(AN0A-AN0B)*SUT)
      AN2A=0.1*Y+0.208*ABS(Y-2.)-
     -     0.233*ABS(Y-5.)+0.125*ABS(Y+10.)+4.
      AN2B=0.5*Y+0.167*ABS(Y)+8.
      AN2=0.5*(AN2A+AN2B+(AN2A-AN2B)*SUT)
      X0A=7.
      X0B=0.5*ABS(Y+5.)-2.
      X0=0.5*(X0A+X0B+(X0A-X0B)*SUT)
      AN1A=(LMLT-20.)**4/8.1E+5+(TAT-16.)**4/256.
      IF (AN1A.GT.100.) AN1A=100.
      AN1A=4.*EXP(-AN1A)
      AN1=AN1A/2*(1+SUT)
      DL1A=10.
      DL1B=10.-0.4*Y+0.01*Y**2
      DL1=0.5*(DL1A+DL1B+(DL1A-DL1B)*SUT)
      DL2A=7.
      DL2B=6.-0.3*Y+0.01*Y**2
      DL2=0.5*(DL2A+DL2B+(DL2A-DL2B)*SUT)
                  IF((X-X0).EQ.0.) GO TO 21
                  SX=(X-X0)/ABS(X-X0)
                  GO TO 22
   21             SX=1.
   22 DL=0.5*(DL1**2*(1+SX)+
     +   DL2**2*(1-SX))
      NW0H=AN0+(AN2-AN0)*EXP(-(X-X0)**2/DL)+AN1
C-------WINTER--- B C A --- KP=3 --------------------------------------
      A=38.+12*SIN((UT-2)*HAD)
      AN0=1.+120./(A-Y)
      DLL=15.+(Y-20.)/6.0
      XL=0.766*SUT*Y-7.66*EXP(-(UT-12)**2/16.)
      AN1A=3.5*EXP(-(20.-Y)/30.)
      AN1B=0.5*(25.-Y)*EXP(-(Y-20.)**2/500.)
      AN1=0.5*(AN1A+AN1B+(AN1A-AN1B)*SIN((UT-3)*HAD))
      ANL=AN1*EXP(-((X-XL)/DLL)**2)
      TATM=16.-2.*SIN((UT-3)*HAD)
      XM=TATM*SIN(ALM*RAD)
      YM=-TATM*COS(ALM*RAD)
      DLM=5.
      AL1M=15.*(9.+10*SIN(0.349*UT)*EXP(-UT/4.3))-25.
      AL2M=180.*SIN(0.349*UT)*EXP(-UT/4.3)-25.
      AN0M=1.+120./(A-YM)+AN1*EXP(-((XM-XL)/DLL)**2)-1.5
      ABCD=((TAT-TATM)/DLM)**4
      IF (ABCD.GT.100.) ABCD=100.
      ANM=AN0M/4.*EXP(-ABCD)*(1+0.6366*ATAN((ALM-AL2M)/2))*
     *    (1-0.6366*ATAN((ALM-AL1M)/2))
      TATE=25.-7.*SUT
      XE=TATE*SIN(ALM*RAD)
      YE=-TATE*COS(ALM*RAD)
      DLE=3.
      AL1E=275.
      AL2E=215.
      AN0E=1.+120./(A-YE)+AN1*EXP(-((XE-XL)/DLL)**2)-1.3
      ABCD=((TAT-TATE)/DLE)**4
      IF (ABCD.GT.100.) ABCD=100.
      A0E=AN0E/4.*EXP(-ABCD)*(1+0.6366*ATAN((ALM-AL2E)/2))*
     *    (1-0.6366*ATAN((ALM-AL1E)/2))
      NW3H=AN0+ANL-ANM-A0E
C-------SUMMER--- B C A ---KP-INPUT------------------------------------
      A1=ZFUN(0.5,-0.5,(KP-1.5)/0.7)
      A2=(1-A1)*(1.+(3.-KP)/20.)
      NSKH=A1*NS0H+A2*NS3H
C-------WINTER--- B C A --- KP-INPUT-----------------------------------
      NWKH=A1*NW0H+A2*NW3H
C-------SUMMER--- F10.7 --- KP-INPUT-----------------------------------
      B=0.6
      NSKF=NSKH*(1+(B-1)*(200-F107)/110.)
C-------WINTER--- F10.7 --- KP-INPUT-----------------------------------
      B=0.1
      NWKF=NWKH*(1.+(B-1.)*(200.-ZFUN(145.,55.,(F107-145.)/27.5))/110.)
C------- TD --- F10.7 --- KP-INPUT-------------------------------------
      AP=1.5
      B=1+0.5*(AP-1)*(1-COS(TD*3.44E-2))
      A1=0.5*(1+COS(TD*1.72E-2))
      A2=1-A1
      NTKF=(A1*NWKF+A2*NSKF)*B
C-------CRITICAL FREQUENCY --- MHz ----------------------------------
      F0F2C=SQRT(NTKF*1.E+5/1.24E+4)
      RETURN
      END
C *********************************************************************
