C
C                    1. MAIN PROGRAM   
C
C                          SMI-96
C
C     STANDARD MODEL OF IONOSPHERE (SMI-96). MARCH 1998.
C     PROGRAM CALCULATES HEIGHT PROFILES OF CONCENTRATION,
C     TEMPERATURE AND EFFECTIVE COLLISION FREQUENCY OF ELECTRONS,
C     ION AND NEUTRAL GAS TEMPERATURE AT ANY LOCATION ON THE EARTH
C     FOR GIVEN HELIO-GEOPHYSICAL CONDITIONS. 
C     MODIFICATIONS [T.L. GULYAEVA, IZMIRAN, AUGUST 1998]: 
C     TOTAL ELECTRON CONTENT TEC FOR [AH,1000 KM],[AH,20000 KM];
C     OUTPUT OF NUMERICAL MAP OF foF2, NmF2 OR TEC FOR GIVEN AREA.
C     SOURCES: SMI-88, INTERNATIONAL REFERENCE IONOSPHERE (IRI-90).        
C     REF.: YU.K.CHASOVITIN, T.L.GULYAEVA, M.G.DEMINOV, S.E.IVANOVA.
C     "RUSSIAN STANDARD MODEL OF IONOSPHERE (SMI)". PREPRINT No.1, 
C      IZMIRAN, MOSCOW, RUSSIA, JAN. 1998; PROC. COST251 WORKSHOP, 
C      MARCH 1998, SIDE, TURKEY. RAL, CHILTON, UK, 161-172, 1998. 
C     ADDRESSES:
C     (1) IZMIRAN, 142092 TROITSK, MOSCOW REGION, RUSSIA
C     (2) SPA "TYPHOON", INSTITUTE OF EXPERIMENTAL METEOROLOGY,   
C         OBNIMSK, KALUGA REGION, PR. LENINA, 82, RUSSIA 
C      
      REAL*8 ATE1(6,6,6),ATE2(6,6,6),A(13),ANE1(3,2,15,13),
     *ANE2(3,9,11)
      REAL LATI,LONGI,MODIP,NMF2,MAGBR,mf(7344),kfm(7392),
     &NMF1,NME,NEI,MM,MLAT,MLONG,FOF2L(2),b0b1(5),wlati(9)
      CHARACTER*3 parname(3)
      DIMENSION XSM(2),JF(10),MM(3),ATN(2),ounu(24,29),outn(24,40),
     &oute(24,29),ATNN(2),CTNN(2),CTN(2),G(144),SDAY(12),OUTF(24),
     &RKFT(11,7,4,4),TA1(24,26),TA2(12,12),TA3(3,13),TA4(4,12),
     &TA5(28,12),TA6(17,12),TA7(17,12),TA8(17,12),hxx(40),OUTH(24),
     &OUTEC1(24),OUTECP(24),TEWWOD(12,9,3,3)
      LOGICAL WINTER,SUMMER,SCHALT,EXT,NIGHT,F1REG,ANF
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK2/B0,B1,C1
     &/BLOCK3/HST1/A1/EH/BLOCK4/G/BLOCD/MONTH,NDATE
     &/BLOCK5/TNX,ATN,CTN,DSL,DGT/BLOCK6/RKFT
     &/BLOCK8/HS,TNHS,XSM,MM/BLO10/BETA,ETA,DELTA,ZETA/B1/ANE1/B2/ANE2
     &/B3/ATE1,ATE2/B6/HMLAT,H0NE,GLT,FGR1,FGR2/B7/A
     &/D6/TA1,TA2,TA3/D25/TA4,TA5/D35/TA6/DMAX/TA7,TA8/BLO11/XHI
     &/amf/mf/akfm/kfm/dem/r,longi,hour,daynr,cn1000,hpr
      EXTERNAL XE2,XE3,XE5,GXXE,TEDER
      DATA wlati /-60.,-40.,-20.,-10.,0.,10.,20.,40.,60./
      DATA SDAY /0.,31.,59.,90.,120.,151.,181.,212.,243.,273.,304.,334./
      data b0b1 /0.755566,0.778596,0.797332,0.812928,0.826146/
      DATA hxx /45.,50.,60.,70.,80.,90.,100.,110.,120.,140.,
     *160.,180.,200.,250.,300.,350.,400.,500.,600.,800.,1000.,
     *1200.,1400.,1600.,1800.,2000.,2500.,3000.,4000.,5000.,6000.,
     *7000.,8000.,9000.,10000.,12000.,14000.,16000.,18000.,20000./
      DATA parname /'Ne ','Te ','Nu '/
      UMR=0.01745329
      PHI=3.1415927
      XFOF2=0.
      XHMF2=0.
      XHI=0.0
C     INPUT OF COEFFICIENTS FOR GEOMAGNETIC FIELD PARAMETERS
      CALL C6KOEF(G)
C     INPUT OF COEFFICIENTS FOR ELECTRON TEMPERATURE
      CALL TEDAT(TEWWOD)
      OPEN (UNIT=08,FILE='BIN08',STATUS='OLD',
     *    ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
      READ (8) ANE1,ANE2
      READ (8) RKFT
      READ (8) ATE1,ATE2
      READ (8) TA1,TA2,TA3,TA4,TA5,TA6,TA7,TA8
      CLOSE (UNIT=08,STATUS='KEEP')
      OPEN (UNIT=10,FILE='BIN10',STATUS='OLD',
     *    ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
      read (10) kfm
      rewind 10
      CLOSE (UNIT=10,STATUS='KEEP')
      OPEN (UNIT=11,FILE='BIN11',STATUS='OLD',
     *    ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
      read (11) mf
      rewind 11
      CLOSE (UNIT=11,STATUS='KEEP')
      OPEN (UNIT=12,FILE='BIN12',STATUS='OLD',
     *    ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
C      OPEN (UNIT=13,FILE='DANTE3',STATUS='OLD',
C     *    ACCESS='SEQUENTIAL',FORM='FORMATTED')
C      read (13,500) tewwod
C  500 format (5x,12f6.0)
C      CLOSE (UNIT=13,STATUS='KEEP')
      OPEN (UNIT=16,FILE='BIN16',STATUS='OLD',
     *    ACCESS='DIRECT',FORM='UNFORMATTED',RECL=11432)
C    1 READ (*,2) LATI,LONGI,R,MONTH,NDATE,IHOUR,MIN,KG,XHI,AH,EH,SH,
C     &(JF(I),I=1,10),JMAG,NTAB1,HEAD,XKP,XKP1,COV
C    2 FORMAT (3F6.1,4I2,I1,3F6.1,F4.1,2X,11I1,2X,I4,8x,A4/
C     &2F3.1,9X,F6.1)
C
      OPEN(33,FILE='MODTG.OUT',STATUS='NEW')
C----------          NEW INPUT FORMAT              ------------------
      WRITE(*,*) 'JF(1:10) ??'
      READ(*,*) (JF(I),I=1,10)
      IOND=JF(1)+JF(2)+JF(3)+JF(4)+JF(9)+JF(10)
      KOMB=JF(5)+JF(6)+JF(7)+JF(8)+JF(9)+JF(10)
      WRITE(*,*) 'JMAG = 0/1 FOR GEOGRAPHIC/GEOMAGNETIC COORDINATES ?'
      READ(*,*) JMAG
C JOUT=0 N(H) PROFILE, JOUT=1 (foF2), JOUT=2 (hmF2), JOUT=3~hup,km (TEC)
      WRITE(*,*) 'JOUT = 0 ~ N(H), 1 ~ foF2, 2 ~ hmF2, 3/hup ~ TEC ?'
      READ(*,*) JOUT
C JMAP=1 foF2; JMAP=3 TEC MAP: [GLAT1,GLAT2],SLAT; [GLON1,GLON2],SLON;
C NOT MORE THAN 24 LONGITUDES FOR EACH LATITUDE; SLON=15 FOR LON=0...360
      WRITE(*,*) 'JMAP=0(NOMAP), 1(foF2), 2(NmF2), 3(TEC1000),4(TECP) ?'
      READ(*,*) JMAP
      IF (JMAP.EQ.0) GOTO 1
C****** NEW INPUT FOR MAP-PRODUCTS.........................
      WRITE(*,*)'MAP: GLAT1,GLAT2,STEP ??'
      READ(*,*) GLAT1,GLAT2,SLAT
      WRITE(*,*)'MAP: GLON1,GLON2,STEP ??'
      READ(*,*) GLON1,GLON2,SLON
    1 LATI=GLAT1
      LONGI=GLON1
      GLON=GLON1
      IF (LONGI.LT.0.) LONGI=LONGI+360.
      WRITE(*,*)'CONTINUE - 0, EXIT - 1 ??'
      READ(*,*) IEXI
      IF(IEXI.EQ.1) GOTO 7097
      WRITE(*,*)'R, YEAR, MONTH, NDATE ??'
      READ(*,*) R,YEAR,MONTH,NDATE
      KG=-1
      IF (INT(YEAR/4.).EQ.(YEAR/4.)) KG=1
      DNMR=365.
      IF (KG.EQ.1) DNMR=366.
      IF (NDATE.EQ.0) NDATE=15
      DAYNR=SDAY(MONTH)+NDATE
      IF ((KG.EQ.1).AND.(MONTH.GT.2)) DAYNR=DAYNR+1
      WRITE(*,*) 'IHOUR ; HOUR+25 FOR UT ?'
      READ(*,*) IHOUR
      MIN=0
      UTHR=IHOUR-25.
      WRITE(*,*) 'XKP,XKP1,COV ??'
      READ(*,*) XKP,XKP1,COV
      STE=1.
C     WRITE(*,*) 'AH, EH, SH ??'
C     READ(*,*) AH,EH,SH
      AH=45
      EH=7000
      SH=100
C*******  AH, EH, SH ARE GIVEN IN OTHER SUBROUTINE
C         HERE THEY ARE ONLY FOR THE FORMAL PRESENTATION
      IF (JOUT.GT.0) GOTO 294
      STE=2.
      WRITE(*,*) 'FOF2, HMF2 or #0 0#'
      READ(*,*) XFOF2,XHMF2
  294 IF (JMAP.GT.0) GOTO 296
      WRITE(*,*)'LATI, LONGI ??'
      READ(*,*) LATI,LONGI
  296 ntab1=0
      head=' SMI'
      ntab2=ntab1+1
C ****** NEXT LATI FOR MAP STARTS HERE ***************************
 1301 DO 4999 I=1,24
      OUTEC1(I)=0.
      OUTECP(I)=0.
 4999 CONTINUE
      IH=0
C      WRITE(*,*)'LATI=',LATI,'LONGI=',LONGI
C******* NEXT LONGI FOR MAP STARTS HERE **************************
C     CALCULATION OF XHI,SUNSET,SUNRISE,COV(F10.7),XHIM ..............
 1300 IF (JMAG.LE.0) GOTO 888
      MLAT=LATI
      MLONG=LONGI
  888 CALL GGM(JMAG,LONGI,LATI,MLONG,MLAT)
      CALL CORDM(LATI,LONGI,MLAT,MLONG,HMLAT,HMLON)
C      WRITE(*,*)'MLAT=',MLAT 
      hprr=cos(mlat*umr)
      hpr=(5.23*hprr*hprr-1.)*6370.+200.
      HKP1=(5.7-0.47*XKP1)*6370.
      HPR=(HPR+HKP1/2.)/1.5
      IF (HPR.GT.20000.) HPR=20000.
      IF (HPR.LT.1000.) HPR=1000.
C      WRITE(*,*)'HPR=',HPR
      do 5000 i=1,40
      if ((hxx(i).gt.1000.).and.(hxx(i).gt.hpr)) go to 5010
 5000 continue
 5010 ktne=i-1
      ktnu=29
      if (ktnu.gt.ktne) ktnu=ktne
      IF (R.GE.0.0) GO TO 3
      R=-409.0+SQRT(95462.6+1123.6*COV)
      IF (R.LT.0.0) R=0.0
    3 IF (COV.LE.0.0) COV=63.75+R*(0.728+R*0.00089)
      call ccirca(month)
      ABSLAT=ABS(LATI)
      ABMLAT= ABS(MLAT)
      ABSHML=ABS(HMLAT)
      IF (UTHR.LT.0.) GOTO 297
      HOUR=UTHR+FLOAT(MIN)/60.0+LONGI/15.
      IF (HOUR.LT.0.) HOUR=HOUR+24.
      IF (HOUR.GE.24.) HOUR=HOUR-24.
      GOTO 8000
  297 HOUR=FLOAT(IHOUR)+FLOAT(MIN)/60.0
 8000 IH=IH+1
      UT=HOUR-LONGI/15.
      IF (UT.LT.0.) UT=UT+24.
      IF (UT.GE.24.) UT=UT-24.
C      WRITE(*,*)'LT=',HOUR,'UT=',UT
      IPP=1
      IF (HOUR.GT.12.) IPP=2
      DAYNS=DAYNR
      IF (LATI.LT.0.) DAYNS=DAYNR+182.
      IF (DAYNS.GT.DNMR) DAYNS=DAYNS-DNMR
      PRK=SPRN(DAYNS)
      SUNDEC=-0.40915*COS(2.0*PHI/365.25*(DAYNR+8))
      Z1=SIN(SUNDEC)*SIN(LATI*UMR)
      Z2=COS(SUNDEC)*COS(LATI*UMR)
      IF (ABS(Z2).GT.0.0) GOTO 120
      SAX=24.0
      IF (Z1.GE.0.0) SAX=0.0
      GOTO 140
  120 IF (ABS(Z1/Z2).LE.1.0) GOTO 510
      SAX=24.0
      IF ((Z1+Z2).GE.0.0) SAX=0.0
      GOTO 140
  510 SAX=12.0-ACOS(-Z1/Z2)/(UMR*15.0)
  140 SUX=24.0-SAX
      XLSTA=15.0*(HOUR-12.0)
      COSXHI=Z1+Z2*COS(XLSTA*UMR)
      XHI=ACOS(COSXHI)/UMR
  503 CALL FIELDG(LATI,LONGI,300.0,XMA,YMA,ZMA,BET,DIP,MODIP)
      MAGBR=ATAN(0.5*TAN(DIP*UMR))/UMR
      SUNDEC=SUNDEC/UMR
      GLT=UT+(HMLON-69.8)/15.
      IF ((GLT.LT.0.).OR.(GLT.GE.24.)) GLT=ABS(ABS(GLT)-24.)
C
C     SPECIFICATION OF REGIONS, TIME OF DAY AND SEASONS.............
C
      SUMMER=.FALSE.
      WINTER=.FALSE.
      NIGHT=.FALSE.
      IF ((MONTH.GT.10).OR.(MONTH.LT.3)) WINTER=.TRUE.
      IF ((MONTH.GT.4).AND.(MONTH.LT.9)) SUMMER=.TRUE.
      IF (LATI.GT.0.0)  GOTO 1111
      EXT=WINTER
      WINTER=SUMMER
      SUMMER=EXT
 1111 IF ((HOUR.GT.SUX).OR.(HOUR.LT.SAX)) NIGHT=.TRUE.
C
C     PARAMETERS OF ELECTRON DENSITY PROFILE.......
C
      ZKP=XKP
      IF (ZKP.GT.6.0) ZKP=6.0
      SGR = SIN(PHI*(GLT-7.0)/12.0)
      FGR1=53.5+4.0*(6.0-ZKP)/6.0+1.5*SGR
      FGR2=60.5+4.0*(6.0-ZKP)/6.0+2.5*SGR
      IF (ABSHML.LE.FGR1) CALL SH1(XHI,DAYNS,PRK,IPP,A)
      IF (ABSHML.GE.FGR2) CALL SH2(XHI,DAYNS,PRK,A)
      IF ((ABSHML.GT.FGR1).AND.(ABSHML.LT.FGR2)) CALL SHINT(XHI,
     &DAYNS,PRK,IPP,A)
      H0NE=SNGL(A(13)+0.0005)
      DH0=H0NE-IFIX(H0NE)
      H0NE=IFIX(H0NE)+0.001*IFIX(DH0*1000)
      IF (ABSHML.GT.FGR1) CALL SMAS(XKP)
C     E AND F REGION PARAMETERS..........................
      CALL RNCOR(XHI,R,MAGBR,COV)
      CALL PMRE (NME,HME,FOE,1,XE5)
      IFO=0
      IF (ABSHML.GE.55.) GO TO 201
      FOF2=F2NW(DAYNR,MLAT,HOUR,LATI,LONGI,MODIP,R)
      IF (ABSHML.LE.45.) GO TO 203
      IFO=IFO+1
      FOF2L(IFO)=FOF2
      XY=(45.-ABSHML)*(ABSHML*(0.002*ABSHML-0.21)+5.4)
  201 IF (ABSHML.GE.75.) GO TO 202
      CALL F0F2AN(HMLAT,LATI,LONGI,DAYNR,UT,R,COV,XKP,XKP1,FOF2)
      IFO=IFO+1
      FOF2L(IFO)=FOF2
      IF (ABSHML.LT.70.) GO TO 203
      XY=(70.-ABSHML)*(ABSHML*(0.016*ABSHML-2.36)+86.8)
  202 CALL SCAP(HMLAT,GLT,UT,XKP,XKP1,COV,DAYNR,FOF2)
      IFO=IFO+1
      FOF2L(IFO)=FOF2
  203 IF (IFO.GT.1) FOF2=FOF2L(2)*XY+FOF2L(1)*(1.-XY)
C -----------   INPUT OF EMPIRICAL VALUE FOF2      -----------------
      IF((XFOF2.GT.0).AND.(IH.EQ.1)) FOF2=XFOF2
      OUTF(IH)=FOF2   
      IF ((MLAT+11.4).GE.0.) GO TO 205
      XLNG=LONGI+180.
      IF (XLNG.GT.360.) XLNG=XLNG-360.
      CALL FIELDG(ABSLAT,XLNG,300.0,XXMA,YYMA,ZZMA,XXBET,XDIP,XMODIP)
      HMF2=HF2OUT(XMODIP,LATI,XLNG,MAGBR,R,HOUR,FOE,FOF2,DAYNR,MLAT,
     &XM3000)
      GO TO 501
  205 HMF2=HF2OUT(MODIP,LATI,LONGI,MAGBR,R,HOUR,FOE,FOF2,DAYNR,MLAT,
     &XM3000)
  501 NMF2=1.24E10*FOF2*FOF2
      IF (JMAP.EQ.2) OUTF(IH)=NMF2
C -----------   INPUT OF EMPIRICAL VALUE HMF2      -------------------
      IF((XHMF2.GT.0).AND.(IH.EQ.1)) HMF2=XHMF2
      OUTH(IH)=HMF2        
       QK=0.8-0.15/(1.+EXP((ABS(MODIP)-30.)/10.))
      QKH=HPOL(HOUR,QK,0.8,SAX,SUX)
      call rogul(dayns,xhi,grat)
      if (night) grat=0.91-hmf2/4000.
      bocnew=hmf2*(1.-grat)
      b0=bocnew/b0b1(1)
      HGF2=QKH*HMF2
      COS2=COS(MLAT*UMR)
      COS2=COS2*COS2
      FLU=(COV-40.0)/30.0
      ETA=0.058798-0.0070305*COS2+FLU*(-0.014065+0.0069724*COS2)+
     &(0.0024287+0.0042810*COS2-0.00015280*FOF2)*FOF2
      ZETA=0.078922-0.0046702*COS2+FLU*(-0.019132+0.0076545*COS2)+
     &(0.0032513+0.0060290*COS2-0.00020872*FOF2)*FOF2
      BETA=-128.03+20.253*COS2+FLU*(-8.0755-0.65896*COS2)+(0.44041
     &+0.71458*COS2-0.042966*FOF2)*FOF2
      Z=EXP(94.45/BETA)
      Z1=Z+1
      Z2=Z/(BETA*Z1*Z1)
      DELTA=(ETA/Z1-ZETA/2.0)/(ETA*Z2+ZETA/400.0)
      B1=3.0
      eh=hxx(22)
      IF (EH.GT.1000.) EH=1000.
      CALL TOPSID(ABMLAT,R,HOUR)
C     F1 LAYER PARAMETERS ............................
      F1REG=.FALSE.
      HMF1=0.
      NMF1=0.
      C1=0.
      XHI0=49.84733+0.349504*ABSLAT
      XHI100=38.96113+0.509932*ABSLAT
      XHIM=(XHI0+(XHI100-XHI0)*R/100.0)
      IF ((XHI.GT.XHIM).OR.NIGHT.OR.WINTER) GOTO 150
      FOF1=FOF1ED(MAGBR,R,COSXHI)
  208 DELA=4.32
      IF (ABS(MODIP).GE.18.0) DELA=1.0+EXP(-(ABS(MODIP)-30.0)/10.0)
      F1REG=.TRUE.
      C1=.09+.11/DELA
      NMF1=1.24E10*FOF1*FOF1
  150 HEF=DTAL(HME,NME)
C
C     CALCULATION OF HMF1 HEIGHT...........................
C
 3801 IF (F1REG) GOTO 924
      HMF1=HMF2
      GOTO 380
  924 call regfa12(hef,hmf2,0.001,nmf1,xe2,schalt,hmf1)
      if (.not.schalt) go to 380
C     WRITE (*,11)
   11 FORMAT(1X,'HMF1 IS NOT EVALUATED BY THE FUNCTION XE2')
c
c     CHANGE B1 AND RE-CALCULATE
c
9244  IF (B1.gt.4.5) GOTO 7398
      B1=B1+0.5
C     WRITE (*,902) b1
  902 FORMAT(6X,'CORR.: B1(OLD)=',F4.1,' B1(NEW)=B1(OLD)-0.5 ')
      ib1=int(b1*2.-5.)
      B0=bocnew/b0b1(ib1)
      GOTO 924
 7398 WRITE (*,9269)
 9269 FORMAT(1X,'CORR: NO F1 REGION, B1=3, C1=0.0')
      HMF1=0.0
      NMF1=0.
      C1=0.0
      B1=3.
      F1REG=.FALSE.
  380 IF (.NOT.F1REG) HMF1=0.
      ZZ1=HMF1
      IF (.NOT.F1REG) HMF1=HGF2
      CALL HSTK(HGF2,ZZ1)
      IF (HME.GT.HST1) CALL PMRE (NME,HME,FOE,1,GXXE)
      cn1000=gxxe(1000.)
      IF (KOMB.LE.0) GO TO 189
C
C     PARAMETERS OF NEUTRAL GAS TEMPERATURE PROFILE.............
C
      ANF=.TRUE.
      Z1=-180.0
  882 TUN=TUNCAL(COV,LATI,SUNDEC,Z1)
      X=0.0045*(TUN-188.0)
      TX=110.5*ALOG(X+SQRT(X*X+1.0))
      TNX=188.0+TX
      ATN(1)=0.63662*TX
      ATN(2)=0.63662*(TUN-TNX)
      GX=1.9*TX/35.0
      CTN(1)=GX/ATN(1)
      CTN(2)=GX/ATN(2)
      IF (.NOT.ANF) GOTO 881
      ANF=.FALSE.
      TNXN=TNX
      ATNN(1)=ATN(1)
      ATNN(2)=ATN(2)
      CTNN(1)=CTN(1)
      CTNN(2)=CTN(2)
      Z1=XLSTA
      GOTO 882
  881 DSLP=(DAYNR-1.0)*6.283185/365.25
      DSL=SIN(DSLP+1.72)
      DGT=57.5*XKP1*(1.0+0.027*EXP(0.4*XKP1))
      DGT=DGT*SIN(MLAT*UMR)**4
      IF ((KOMB-JF(6)).LE.0) GO TO 189
C
C     PARAMETERS OF ELECTRON TEMPERATURE PROFILE........................
C
      IF (ABMLAT.LT.65.) CALL RTE(ABMLAT,XHI)
      CALL TEK(MLAT,HOUR,SAX,SUX,DAYNR,MAGBR,R)
C    CALCULATE  < 4000 KM ONLY FOR MARCH, JULY, SEPT. AND DEC.;
C    GEOGRAPH. LONGI: 30, 150, 270; LATI: -60, -40, ......
      if (month.eq.12) ise=1
      if (month.eq.6) ise=2
      IF (LATI.GE.0.) GO TO 4998
      IF (MONTH.EQ.6) ISE=1
      IF (MONTH.EQ.12) ISE=2
 4998 if ((month.eq.3).or.(month.eq.9)) ise=3
      if (longi.eq.30.) ilo=1
      if (longi.eq.150.) ilo=2
      if (longi.eq.270.) ilo=3
      do 20 kk=1,9
      if (lati.eq.wlati(kk)) go to 33
   20 continue
   33 ila=kk
      t3000=tewwod(ih,ila,ilo,ise)
      if (xhi.ge.100.) te3000=t3000+(r-100.)*3.08
      if (xhi.lt.100.) te3000=t3000+(r-100.)*6.16
      if (xhi.le.80.) te3000=t3000+(r-100.)*9.25
      tn1400=tn(1400.,atn,ctn,tnx,dsl,dgt)
      call tegt(1400.,te1400)
      if (te1400.lt.tn1400) te1400=tn1400
      if (te3000.lt.te1400) te3000=te1400
      pinta=(te3000-te1400)/1600.
      IF ((JF(7)+JF(8)).LE.0) GO TO 240
C
C     PARAMETERS OF ION TEMPERATURE PROFILE.................
C
      XSM(1)=430.0
      TNN4=TN(XSM(1),ATNN,CTNN,TNXN,DSL,DGT)
      TNN6=TN(600.0,ATNN,CTNN,TNXN,DSL,DGT)
      X1=(TNN6-TNN4)/170.0
      MM(2)=HPOL(HOUR,3.0,X1,SAX,SUX)
      Z1=EXP(-0.09*MLAT)
      X4=1240.0-1400.0*Z1/((1.0+Z1)*(1.0+Z1))+3.08*(R-80.0)
      X2=ABMLAT*(0.47+ABMLAT*0.024)*UMR
      X3=COS(X2)
      TIN=1200.0-300.0*SIGN(1.0,X3)*SQRT(ABS(X3))+3.08*(R-80.0)
      YSM2=TIN
      IF (X4.GE.TIN) YSM2=HPOL(HOUR,X4,TIN,SAX,SUX)
      CALL TEGT(XSM(1),Z1)
      Z2=TN(XSM(1),ATN,CTN,TNX,DSL,DGT)
      IF (Z1.GT.Z2) GO TO 280
      Z1=Z2
      YSM2=Z2
      HS=430.0
      GO TO 250
  280 IF ((YSM2.LE.Z2).OR.(YSM2.GE.Z1)) YSM2=(Z1+Z2)/2.0
      CALL REGFA12(130.0,430.0,0.01,YSM2,TEDER,SCHALT,HS)
      IF (.NOT.SCHALT) GOTO 250
      HS=130.0
  250 TNHS=TN(HS,ATN,CTN,TNX,DSL,DGT)
      MM(1)=DTNDH(HS,ATN,CTN,DSL,DGT)
      MM(3)=MM(2)
      XSM(2)=XSM(1)
C
C     SMOOTHING ION TEMPERATURE PROFILE IN ORDER TI WAS...
C     NOT GREATER THAN TE AT HEIGHT HSM
C
      HSM=1000.
      CALL TEGT(HSM,TESM)
      TNSM=TN(HSM,ATN,CTN,TNX,DSL,DGT)
      IF (TESM.LT.TNSM) TESM=TNSM
      IF (TESM.GE.TI(HSM)) GO TO 240
      CALL TEGT(HSM,TSM2)
      CALL TEGT(HSM-1.0,TSM1)
      MM(3)=TSM2-TSM1
      XSM(2)=(TESM-10.-YSM2+MM(2)*XSM(1)-MM(3)*HSM)/(MM(2)-MM(3))
      IF (XSM(2).LT.450.0) XSM(2)=450.0
      IF (XSM(2).GT.980.0) XSM(2)=980.0
  240 IF ((JF(9)+JF(10)).LE.0) GO TO 189
C
C     CALCULATION OF ELECTRON COLLISION FREQUENCY
C
      CALL FKNUH(ABSLAT)
      PN120=XE5(120.)
      CALL FITE(MLAT,HOUR,SAX,SUX,120.,PT120,TE2)
      CALL J77(LATI,LONGI,MLAT,DAYNR,HOUR,COV,COV,XKP1)
C
C     CALCULATION OF PROFILES OF REQUIRED PARAMETERS.................
C
  189 HMIN=H0NE
      IF (JOUT.LT.3) GOTO 188
C
C   CALCULATION OF TEC*10^15, M-3: TEC1 BELOW 1000 KM, TECP<=20000 KM
C
      HUP=HPR
      X=50.
      YNE2=0.
      STEP=10.
      IF (JOUT.GT.3) HUP=FLOAT(JOUT)
      DO 190 I=1,1005
      YNE1=YNE2
      IF (X.GE.200.) STEP=20.
      X=X+STEP
      IF (X.GT.HUP) GO TO 191 
      IF (X.LT.HMIN) GO TO 190
      YNE2=GXXE(X)
      IF (YNE1.EQ.0) GO TO 190
      OUTECP(IH)=OUTECP(IH)+(YNE1+YNE2)*STEP/2.
      IF (X.EQ.1000.) OUTEC1(IH)=OUTECP(IH)*1000.
  190 CONTINUE
  191 OUTECP(IH)=OUTECP(IH)*1000. 
  188 IF (JMAP.EQ.0) GOTO 301
      GLON=GLON+SLON
      LONGI=GLON
      IF (LONGI.GE.360.) LONGI=LONGI-360.
      IF (GLON.LE.GLON2) GO TO 1300
C GOTO OUTPUT OF MAP LINE FOR FIXED LATITUDE
      GO TO 370
  193 GLON=GLON1
      LONGI=GLON
      IF (LONGI.LT.0) LONGI=LONGI+360.
      LATI=LATI+SLAT
      IF (LATI.LE.GLAT2) GOTO 1301
      GO TO 1
C 
  192 do 300 i=1,ktne
      X=hxx(i)
      if (x.lt.hmin) NEI=0.0
      if (x.ge.hmin) NEI=GXXE(X)
      outn(ih,i)=nei
 7108 if (komb.le.0) go to 300
      IF (i.gt.ktnu) GOTO 300
      if (x.gt.1400.) go to 7211
      IF (X.GE.90.) GO TO 7109
      oute(ih,i)=0.
      GOTO 7118
 7109 X1=TN(X,ATN,CTN,TNX,DSL,DGT)
      CALL TE(MLAT,HOUR,SAX,SUX,X,TE1,TE2)
      IF ((TE1.GT.0.).AND.(TE1.LT.X1)) TE1=X1
      IF ((TE2.GT.0.).AND.(TE2.LT.X1)) TE2=X1
      IF ((TE2.LT.0.).AND.(X.GE.155.).AND.(ABMLAT.LT.65.)) TE2=TE1
      IF (TE2.LT.X1) TE2=X1
      oute(ih,i)=te2
      IF ((ABSLAT.GE.70.).OR.((LATI.EQ.60.).AND.(LONGI.EQ.270.)).OR.
     *((LATI.EQ.-60.).AND.(LONGI.EQ.150.))) oute(ih,i)=te1
      go to 7118
 7211 te1=te1400+pinta*(x-1400.)
      oute(ih,i)=te1
 7118 if (oute(ih,i).lt.0.) oute(ih,i)=0.
      if (x.gt.1000.) go to 330
      PNUE=HNU(X,DAYNS,NEI,TE1,PN120,PT120)
      ounu(ih,i)=pnue
      go to 300
  330 ste1=sqrt(te1)
      ounu(ih,i)=54.*nei*1.0e-06/(te1*ste1)
  300 continue
  301 HOUR=HOUR+STE
      if (hour.le.23) go to 8000
      if (komb.le.0) go to 310
      nte=0
      do 350 ll=1,ktnu
      if (oute(1,ll).gt.0.) go to 360
  350 nte=nte+1
  360 ktte=ktnu-nte
  310 ILAT=IFIX(LATI)
      ILON=IFIX(LONGI)
C
C     OUTPUT OF RESULRS..............................................
C **********************  OUTPUT                --------------------
  370 IF (JMAP.EQ.0) GO TO 371
      IF (JMAP.EQ.1) GO TO 374
      IF (JMAP.GT.2) GO TO 375
C     NmF2 UNITS 10**10, M-3 
      WRITE(33,5557) (INT(OUTF(K)/(1E+10)+0.5),K=1,24)
      IF (JMAP.EQ.2) GO TO 193
  375 IF (JMAP.EQ.3) GO TO 372
      IF (JMAP.EQ.4) GO TO 373
  371 IF (JOUT.LT.3) GO TO 5558 
C     TEC UNITS 10**15, M-2
      WRITE(33,*)'hup=',HUP
 5557 FORMAT (2X,24(I4,1X))
  372 WRITE(33,5557) (INT(OUTEC1(K)/(1E+15)+.5),K=1,24)
      IF (JMAP.EQ.3) GO TO 193
  373 WRITE(33,5557) (INT(OUTECP(K)/(1E+15)+.5),K=1,24)
      IF (JMAP.EQ.4) GO TO 193
      IF (JOUT.GT.2) GO TO 1
 5558 IF (JOUT.EQ.0) GO TO 5559
      IF (JOUT.EQ.2) GO TO 376
C     FOF2*10, MHz
  374 WRITE(33,5557) (INT(OUTF(K)*10+0.5),K=1,24)
      IF (JMAP.EQ.1) GO TO 193
      IF (JOUT.EQ.1) GO TO 1
  376 WRITE(33,5557) (INT(OUTH(K)),K=1,24)
      IF (JOUT.EQ.2) GO TO 1
 5559 WRITE(33,*)'FOF2=',FOF2,' HMF2=',HMF2,' K1=',XKP,' K2=',XKP1
      WRITE (33,5556) ntab1,parname(1),ktne,int(r), month,ilat,ilon
 5556 FORMAT (2X,'T.',i4,2x,a3,'(',i2,')',i5,i4,2i5)
      DO 7040 k=1,ktne
 7040 WRITE (33,7888) int(hxx(k)),(OUTN(L,k),L=1,12)
      if (komb.le.0) go to 1
      ntab2=ntab1+1
      WRITE (33,5556) ntab2,parname(2),ktte,int(r), month,ilat,ilon
      DO 7050 k=1,ktte
 7050 WRITE (33,7886) int(hxx(k+nte)),(int(oute(l,k+nte)),l=1,12)
 7886 format (2x,13(i6,1x))
      ntab3=ntab1+2
      WRITE (33,5556) ntab3,parname(3),ktnu,int(r), month,ilat,ilon
      do 7777 m=1,ktnu
 7777 write (33,7888) int(hxx(m)),(ounu(l,m),l=1,12)
 7888 format (2x,0pi6,12(1x,1pe8.2))
      GO TO 1
 7097 CONTINUE
      CLOSE (UNIT=12,STATUS='KEEP')
      CLOSE (UNIT=16,STATUS='KEEP')
      CLOSE(33)
      STOP
      END
C
C      2. SUBROUTINES FOR ELECTRON CONCENTRATION PROFILES
C
      REAL FUNCTION GXXE(H)
C     FULL ELECTRON DENSITY PROFILES AT RANGE OF ALTITUDES 
C     FROM AH TO 20000 KM
      REAL NMF2
      COMMON/BLOCK1/HMF2,NMF2,HMF1/BLOCK3/H1
      COMMON /BLKS/Y1,Y11,H2,Y2,Y21,KEY,X0
      IF(H.GT.1000.) GOTO 10
      IF(H.LT.HMF2) GOTO 100
      GXXE=XE1(H)
      RETURN
  100 IF(H.LT.HMF1) GOTO 200
      GXXE=XE2(H)
      RETURN
  200 IF(H.LT.H2) GOTO 300
      GXXE=XE3(H)
      RETURN
  300 IF (H.LE.H1) GO TO 400
      GXXE=XE4(H)
      RETURN
  400 GXXE=XE5(H)
      RETURN
   10 GXXE=XE6(H)
      RETURN
      END
C
      FUNCTION XE6(X)
C     ELECTRON DENSITY PROFILE ABOVE 1000 KM
      integer  ND
      real N6370,L,LT
C      real A0,A1,A2,A3,AL,AL2,BL,BL2,H,DN
C      real RE, RAD
      COMMON /DEM/W,GLON,HOUR,DAYNR,CN1000,HPR
      parameter ( PI=3.14159265, RE=6370., RAD=PI/180.)
C      IF (H.GT.HPR) XE6=0.
C      RETURN
      LT=HOUR
      ALT=X
      ND=IFIX(DAYNR)
      L= 1.0 + ALT/RE
c
      A0 = (1.0+0.7*COS(RAD*(GLON+70.)))*(1.+COS(2*PI*(ND+16)/365.25))
      A1 = 1.-0.10*COS(PI*(LT-4.)/12.)
      A2 = 1.+0.2*SQRT(W)
      A3 = 1.+0.001*W
      AL = 1.-EXP(-0.04*L**4)
      AL2 = 1.-EXP(-0.04*2**4)
      BL = 0.5*A0*AL-0.7*L*EXP(0.1*A0*AL)
      BL2 = 0.5*A0*AL2-0.7*2*EXP(0.1*A0*AL2)
      N6370 = 3.0E9*A1*A2*EXP(A3*BL2)
      H = 1342.5/ALOG(CN1000/N6370)
      if (ALT.le.6370.) then
         DN = CN1000*EXP((1000.-ALT)/H/L/L)
      else
         DN = 3.0E9*A1*A2*EXP(A3*BL)
      endif
      XE6=DN
      RETURN
      END
C
      SUBROUTINE REGFA12(X11,X22,EPS,FW,F,SCHALT,X)
C     LOGIC SUBROUTINE TO CALCULATE SUCH X, FOR WHICH 
C     F(X)-FW-0, X11,X22 - STARTING VALUES. CALCULATION IS TERMINATED
C     WHEN X-INTERVAL BECOME LESS THAN EPS. IF
C     SING(F(X1)-FW)=SING(F(X2)-FW), THEN SCHALT=.TRUE.
      LOGICAL L1,LINKS,K,SCHALT
      SCHALT=.FALSE.
      EP=EPS
      X1=X11
      X2=X22
      F1=F(X1)-FW
      F2=F(X2)-FW
      K=.FALSE.
      NG=2
      LFD=0
      IF(F1*F2.LE.0.0) GOTO 200
      X=0.0
      SCHALT=.TRUE.
      RETURN
200   X=(X1*F2-X2*F1)/(F2-F1)
      GOTO 400
300   L1=LINKS
      DX=(X2-X1)/NG
      IF(.NOT.LINKS) DX=DX*(NG-1)
      X=X1+DX
400   FX=F(X)-FW
      LFD=LFD+1
      IF(LFD.GT.20) THEN
      EP=EP*10.
      LFD=0
      ENDIF
      LINKS=(F1*FX.GT.0.0)
      K=.NOT.K
      IF(LINKS) THEN
      X1=X
      F1=FX
      ELSE
      X2=X
      F2=FX
      ENDIF
      IF(ABS(X2-X1).LE.EP) GOTO 800
      IF(K) GOTO 300
      IF((LINKS.AND.(.NOT.L1)).OR.(.NOT.LINKS.AND.L1)) NG=2*NG
      GOTO 200
800   RETURN
      END
C ******************************************************************
C ******************************************************************
C
C
C *********************************************************************
C *********************************************************************
C *********************************************************************
