C**********************************************************                     
	real*8 function PLAS(ALT)
CC
CC==== IZMIRAN PLASMASPHERE ELECTRON DENSITY PROFILE ABOVE h05top
CC     Ref. Gulyaeva T.L., X.Huang, and B.W. Reinisch. Plasmaspheric extension of 
CC     topside electron density profiles. Adv. Space Res., 29, No.6, 825-831, 2002. 
CC
CC     T.Gulyaeva............................., July 2004 - September 2013.
CC Formulation is changed fitting plasmasphere model to topside ionosphere model at
CC the Topside Transition Ionosphere Height
CC HEITOP at, Electron Density ELTOP = 0.5*NmF2.
CC Output : electron density, PLAS, [m^-3], at altitude X  [km] above HEITOP
CC 
C
C  Parameters used by function PLAS1:
C
C  W = R12 = Rzs 12-months smoothed sunspot number
C  GLON - geographic longitude
C  HOUR - local time, UT - universal time
C  LDAYNR - day-of-year 
C  ALON, BLON - corrected geomagnetic coordinates
C  HEITOP = h05top - altitute in the topside at Ne05 = 0.5* NmF2
C  X - altitude in the range:  [h05top, 36,000 km]
C
	implicit real*8 (a-h,o-z)
      INTEGER  ND
      real*8 N6370,L,LT
	real*8 eltop
	COMMON /bplas/w,glat,glon,put,ldaynr,heitop,eltop
      PARAMETER ( PI=3.14159265, RE=6370., RAD=PI/180.)
C
C Specify corrected geomagnetic coordinates alon, blon
C SMI SUBR. ABLON TO PREPARE FOR PLASMASPHERE MODEL
      alon=365.
      blon=365.
      if (((glon.ge.130.).and.(glon.le.178.)).or.((glon.ge.330.).
     &and.(glon.le.340.))) goto 209
      CALL ABLON(GLAT,GLON,ALON,BLON)
  209 continue

	 HHUP=HEITOP
      ND=LDAYNR
	LT=PUT+GLON/15.0
	if (LT.ge.24.) LT=LT-24.0
	if (LT.lt.0.) LT=LT+24.0
      L= 1.0 + ALT/RE
c
	A1 = 1.-0.1*COS(PI*(LT-4.)/12.)
      A2 = 1.+0.2*SQRT(W)
      A3 = 1.+0.001*W
      AL = 1.-EXP(-0.04*L**4)
      AL2 = 1.-EXP(-0.04*2**4)
C !!!!
	c1=3.0E+09
C !!New!!    CORRECTION: C1=3E+09 AS UPPER LIMIT OF COEFFICIENT
	C1=.0002*ELTOP*(RE-HHUP)
	IF (C1.GT.3.0E+09) C1=3.0E+09
C!!!!
      if (alon.le.360.) goto 10
      A0 = (1.0+0.7*COS(RAD*(GLON+70.)))*(1.+COS(2*PI*(ND+16)/365.25))
      BL = 0.5*A0*AL-0.7*L*EXP(0.1*A0*AL)
      BL2 = 0.5*A0*AL2-1.4*EXP(0.1*A0*AL2)
d-      N6370 = 3.0E9*A1*A2*EXP(A3*BL2)
      N6370 = c1*A1*A2*EXP(A3*BL2)
d+!!! old formula:      H = 1342.5/ALOG(CN1000/N6370) IS REPLACED BY :
	H=(RE-HHUP)*.25/DLOG(ELTOP/N6370)
      if (ALT.le.6370.) then
C!!! old formula:    DN = CN1000*EXP((1000.-ALT)/H/L/L) IS REPLACED BY :
	   DN=ELTOP*EXP((HHUP-ALT)/H/L/L)
      else
C!!! old formula:    DN = 3.0E9*A1*A2*EXP(A3*BL) IS REPLACED BY :
	   DN=C1*A1*A2*EXP(A3*BL)
      endif
      Plas=DN
      RETURN
   10 a0a= (1.0+0.7*COS(RAD*(ALON+70.)))*(1.+COS(2*PI*(ND+16)/365.25))
      b0b= (1.0+0.7*COS(RAD*(BLON+70.)))*(1.+COS(2*PI*(ND+16)/365.25))
      bla= 0.5*a0a*AL-0.7*L*EXP(0.1*a0a*AL)
      blb= 0.5*b0b*AL-0.7*L*EXP(0.1*b0b*AL)
      bl2a= 0.5*a0a*AL2-0.7*2*EXP(0.1*a0a*AL2)
      bl2b= 0.5*b0b*AL2-0.7*2*EXP(0.1*b0b*AL2)
      drea = 3.0E9*A1*A2*EXP(A3*bl2a)
      dreb = 3.0E9*A1*A2*EXP(A3*bl2b)
      drea=dlog10(drea)
      dreb=dlog10(dreb)
      delon=blon-alon
      if (delon.le.0.) delon=delon+360.
      deglon=glon-alon
      if (deglon.le.0.) deglon=deglon+360.
      dne=drea+(dreb-drea)*deglon/delon
      N6370=10.**dne
C!!!      H = 1342.5/ALOG(CN1000/N6370)
	H=(RE-HHUP)*.25/DLOG(ELTOP/N6370)
      if (ALT.le.6370.) then
C!!!         DN = CN1000*EXP((1000.-ALT)/H/L/L)
	   DN=ELTOP*EXP((HHUP-ALT)/H/L/L)
      else
C!!!         dna = 3.0E9*A1*A2*EXP(A3*bla)
	         dna = C1*A1*A2*EXP(A3*bla)
         dna=dlog10(dna)
C!!!         dnb = 3.0E9*A1*A2*EXP(A3*blb)
         dnb = C1*A1*A2*EXP(A3*blb)
         dnb=dlog10(dnb)
         dnx=dna+(dnb-dna)*deglon/delon
         dn=10.**dnx
      endif
      Plas=DN
      RETURN
      END
C
      SUBROUTINE ablon(xlati,xlongi,alonn,blonn)
C==== GRID POINTS FOR THE FUNCTION PLAS(alt,...) 
      implicit real*8 (a-h,o-z)
      DIMENSION xlat(37),xlon(37),zlon(6)
      DATA xlat /10.62,10.43,10.06,9.60,9.10,8.76,8.60,8.74,9.29,9.82,
     &9.83,9.27,8.38,7.48,7.07,7.05,6.25,4.07,1.76,0.36,-0.42,-1.37,
     &-2.84,-4.01,-4.15,-4.37,-6.34,-9.73,-12.75,-14.46,-14.12,-10.41,
     &-3.90,2.66,7.44,9.95,10.62/
      DATA xlon /0.,10.,20.,30.,40.,50.,60.,70.,80.,90.,100.,110.,120.,
     &130.,140.,150.,160.,170.,180.,190.,200.,210.,220.,230.,240.,250.,
     &260.,270.,280.,290.,300.,310.,320.,330.,340.,350.,360./
      do 1 i=1,36
      k=i
      if ((xlongi.ge.xlon(i)).and.(xlongi.lt.xlon(i+1))) goto 2
    1 CONTINUE
    2 fimeq=xlat(k)+(xlat(k+1)-xlat(k))*(xlongi-xlon(k))*0.1
      if ((xlongi.lt.178.).or.(xlongi.gt.330.)) goto 3
      if (xlati.le.(fimeq+1.)) goto 11
      if (xlati.lt.2.6) goto 4
      alonn=178.
      blonn=330.
      goto 11
    4 nl=0
      do 5 i=1,36
c      k=i
      if (((xlati.ge.xlat(i)).and.(xlati.lt.xlat(i+1))).or.
     &((xlati.le.xlat(i)).and.(xlati.gt.xlat(i+1)))) goto 15
      goto 5
   15 nl=nl+1
      zlon(nl)=xlon(i)+10.*(xlati-xlat(i))/(xlat(i+1)-xlat(i))
    5 continue
      alonn=zlon(1)
      blonn=zlon(2)
      goto 11
    3 if (xlati.ge.(fimeq-1.)) goto 11
      if (xlati.gt.7.6) goto 6
      alonn=340.
      blonn=130.
      goto 11
    6 nl=0
      do 7 i=1,36
c      k=i
      if (((xlati.ge.xlat(i)).and.(xlati.lt.xlat(i+1))).or.
     &((xlati.le.xlat(i)).and.(xlati.gt.xlat(i+1)))) goto 17
      goto 7
  17  nl=nl+1
      zlon(nl)=xlon(i)+10.*(xlati-xlat(i))/(xlat(i+1)-xlat(i))
    7 continue
      if ((xlongi.ge.zlon(1)).and.(xlongi.le.130.)) goto 8
      alonn=zlon(nl)
      blonn=zlon(1)
      goto 11
    8 continue
      do 9 i=1,nl
      k=i
      if ((xlongi.ge.zlon(i)).and.(xlongi.le.zlon(i+1))) goto 10
    9 continue
   10 alonn=zlon(k)
      blonn=zlon(k+1)
   11 RETURN
      END
C
C
C**********************************************************
C
      integer function ndoy(iyr,imn,idy)
C To define day-of-year
C
	       DIMENSION IM(12)
	integer*2 iyr,imn,idy
	DATA IM/31,28,31,30,31,30,31,31,30,31,30,31/
	z1=iyr/4.0
      jz1=int(z1)*4
      IF(jz1.EQ.iyr) THEN
               IM(2)=29
        ELSE
                IM(2)=28
	       ENDIF
C Day-of-year LDA1
		mosum=0
      if(imn.gt.1) then
         do 1234 i=1,imn-1
1234    mosum=mosum+im(i)
         endif
      ndoy=mosum+idy	
	END FUNCTION
C ===============================================================
C New:	 T.L. Gulyaeva.................................Sep 30, 2013
	subroutine prepPlas(hm,aep,bb)
C Determine height h05top(0.5*NmF2) for NeQuick Plasmasphere starting height:
      implicit real*8 (a-h,o-z)
      real*8 NeNeQ
	dimension hm(3),aep(3),bb(6)
	COMMON /bplas/rzs,plat,plong,put,ldaynr,heitop,eltop 
C
C Start iterations to obtain h05top:
c	eltop=aep(1)*1E+11/8.   ! 0.5*NmF2
	htop=hm(1)      ! start from hmF2
	dh=100.
	iflag=0
    1  htop=htop+dh
    	e1=NeNeQ(htop,hm,aep,bb)
	if (iflag.gt.3) goto 2
	if (e1.lt.eltop) then
	htop=htop-dh
	dh=dh/10.
	iflag=iflag+1
	goto 1
	                  else
	goto 1
	endif
    2	heitop=htop
      RETURN
	end
C***********************************************************************